#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_AccessLoggableObject.py,v 1.5 2003/04/01 17:45:34 wankyu Exp $
Description

    NeoPortal Access Loggable Object can log its daily accesses
 
"""

from Globals import InitializeClass
from BTrees.OOBTree import OOBTree
from AccessControl import ClassSecurityInfo, getSecurityManager
from DateTime.DateTime import DateTime

from Products.NeoPortalLibrary.NP_AccessTraceableObject import NeoPortalAccessTraceableObject

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalAccessLoggableObject( NeoPortalAccessTraceableObject ):
    """
        Mix-in that allows access logging
    """
    
    neoportal_access_loggable_content = 1
    
    _np_access_log_container_name = 'neoportal_access_log'
    
    security = ClassSecurityInfo()
    
    security.declarePublic( 'accessLoggingEnabled' )        
    def accessLoggingEnabled( self ):
        """
           Tell if content access logging is enabled
        """
        
        if not getattr( self, 'neoportal_access_loggable_content', 0 ): return 0
        
        if hasattr( self, 'np_access_logging_enabled' ):
            return self.np_access_logging_enabled
        else:
            try:
                return self.neoportal_properties.isContentAccessLoggable( self )
            except:
                return 0

    security.declareProtected( NeoPortalModifyPortalContent, 'getNeoPortalAccessTraceKey' )        
    def getNeoPortalAccessLogTraceKey( self ):
        """
            Return how to trace a user access
        """

        return self.getNeoPortalAccessTraceKey( container_name=self._np_access_log_container_name )

    security.declarePublic( 'getNeoPortalReadCount' )
    def getNeoPortalReadCount( self, access_trace_key=None ):
        """
            Return read count
        """     
        
        # if 'access_trace_key' is not specified, return the sum of all daily counts
        container = self.getNeoPortalAccessLogContainer()
        
        if access_trace_key is None:
            read_counts = container.values()
            if not read_counts: 
                self.np_read_count = 0
                return self.np_read_count
        
            self.np_read_count = reduce( lambda x,y: x+y, read_counts )
            return self.np_read_count
        else:
            if container.has_key( access_trace_key ):
                return container[ access_trace_key ]
            else:
                return 0

    security.declarePublic( 'getNeoPortalAverageAccessPerDay' )
    def getNeoPortalAverageAccessPerDay( self ):
        """
           Return the average content rating
        """
        read_counts = self.getNeoPortalAccessLogContainer().values()
        if not read_counts:
            return 0
            
        return float(reduce(lambda x,y: x+y, read_counts))/float(len(read_counts))

    security.declarePublic( 'upNeoPortalReadCount' )
    def upNeoPortalReadCount( self ):
        """
            Increase today's read count
        """     
        
        if not self.accessLoggingEnabled( ):
            return
            
        # CMF? Check review status first: skip logging unpublished content
        try:
            review_state = self.portal_workflow.getInfoFor( self, 'review_state', '')
            if review_state and review_state.lower() != 'published' and review_state.lower() != 'public':
                return
        except:
            pass
        
        container = self.getNeoPortalAccessLogContainer()
        access_trace_key=[]
        if 'Date' in self.getNeoPortalAccessLogTraceKey():
            access_trace_key.append( DateTime().strftime('%Y-%m-%d') )
        
        if 'IP Address' in self.getNeoPortalAccessLogTraceKey():
            access_trace_key.append( self._getRemoteIP() )

        if 'Username' in self.getNeoPortalAccessLogTraceKey():
            user = self.getAuthenticatedNeoPortalUser()
            
            if user is None or not hasattr( user, 'getUserName' ):
                username = 'Anonymous'
            else:
                username = user.getUserName()    
                        
            access_trace_key.append( username )
        
        access_trace_key = self.getNeoPortalAccessTraceKeySplitter().join( access_trace_key )        
        
        if container.has_key( access_trace_key ):
            container[ access_trace_key ] = container[ access_trace_key ] + 1
        else:
            container[ access_trace_key ] = 1
        
        if hasattr( self, 'updateNeoPortalElementContainerCatalogIndex'):
            self.updateNeoPortalElementContainerCatalogIndex( self )

    security.declarePrivate( 'getNeoPortalAccessLogContainer' )          
    def getNeoPortalAccessLogContainer( self ):
        """
            Return container that holds access log
        """
        
        return self.getNeoPortalAccessTraceContainer( container_name=self._np_access_log_container_name )

    getNeoPortalAccessLog = getNeoPortalAccessLogContainer

    security.declareProtected( NeoPortalView, 'getNeoPortalAccessLogKeys' )          
    def getNeoPortalAccessLogEntries( self ):
        """
            Return log entry keys
        """
        return self.getNeoPortalAccessLogContainer().keys()
    
    security.declareProtected( NeoPortalView, 'getNeoPortalNumAccessLogEntries' )          
    def getNeoPortalNumAccessLogEntries( self ):
        """
           Return the number of entries in the log
        """
        return len( self.getNeoPortalAccessLogContainer() )

    security.declareProtected( NeoPortalModifyPortalContent, 'purgeNeoPortalAccessLog' )        
    def purgeNeoPortalAccessLog( self, purge_days=0, REQUEST=None ):
        """
            Purge 'purge_days' old logs
        """
        
        # if 'purge_days' is not specified, purge 'em all
        
        if not purge_days or not int( purge_days ): 
            self.initNeoPortalAccessTraceContainer( container_name=self._np_access_log_container_name )
        else:    
            container = self.getNeoPortalAccessLogContainer()
            
            today = DateTime()
            
            for entry in container.keys( ):
                entry = entry.split( self.getNeoPortalAccessTraceKeySplitter() )[0]
                
                if int(today - DateTime(entry)) > int( purge_days ):
                    del container[entry]

        if hasattr( self, 'addToNeoPortalElementContainerCatalog'):
            self.addToNeoPortalElementContainerCatalog( self )
                    
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message="Log purged." )
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=%s" % message
                )
            
InitializeClass( NeoPortalAccessLoggableObject )

