#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_AccessTraceableObject.py,v 1.2 2003/03/30 13:07:31 wankyu Exp $
Description

    NeoPortal AccessTraceable Object can trace users access and block duplicate operations
 
"""

from Globals import InitializeClass
from BTrees.OOBTree import OOBTree
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from DateTime.DateTime import DateTime

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_TranslatableContent import NeoPortalTranslatableContent

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalAccessTraceableObject( NeoPortalBaseItem, 
                                      NeoPortalTranslatableContent ):
    """
        Mix-in that allows user access to get logged by IP, username or both
    """
    
    neoportal_access_traceable_content = 1
    
    neoportal_access_trace_keys = [ 'Date', 'IP Address','Username' ]  # date, ip, username
    
    neoportal_access_trace_key_splitter = '|'

    security = ClassSecurityInfo()    

                            
    security.declarePrivate( 'getNeoPortalAccessTraceContainer' )     
    def getNeoPortalAccessTraceContainer( self, container_name='default' ):
        """
            Return container that hold access traces
        """
        if not hasattr( aq_base( self ), '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()
        
        if not self._np_access_traces_containers.has_key( container_name ):
            self.initNeoPortalAccessTraceContainer( container_name=container_name )        

        container, access_trace_key = self._np_access_traces_containers[ container_name ]
        return container

    security.declarePublic( 'getNeoPortalAccessTraceKeySplitter' )     
    def getNeoPortalAccessTraceKeySplitter( self ):
        """
            Return key splitter
        """
        
        return self.neoportal_access_trace_key_splitter


    security.declareProtected( NeoPortalModifyPortalContent, 'initAllNeoPortalAccessTraceContainers' )        
    def initAllNeoPortalAccessTraceContainers( self, REQUEST=None ):
        """
            Initialize all containers
        """

        self._np_access_traces_containers = {}            

        if REQUEST is not None:
            message = self.getNeoPortalMessage( message="All containers initialized." )
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=%s" % message
                )                
                
    security.declareProtected( NeoPortalModifyPortalContent, 'initNeoPortalAccessTraceContainer' )        
    def initNeoPortalAccessTraceContainer( self, container_name='default', access_trace_key=None, REQUEST=None ):
        """
            Initialize the container
        """
        if not access_trace_key:
            access_trace_key = self.neoportal_access_trace_keys

        # backward compatibility
        if hasattr( aq_base( self ), '_np_content_ratings' ) and container_name == 'neoportal_content_ratings':
            container = ( self._np_content_ratings, ['IP Address',] )
            del self._np_content_ratings
        elif hasattr( aq_base( self ), '_np_access_log' ) and container_name == 'neoportal_access_log':
            container = ( self._np_access_log, ['Date',] )
            del self._np_access_log        
        else:
            container = ( OOBTree(), access_trace_key )

        if not hasattr( aq_base( self ),  '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()
        
        self._np_access_traces_containers[ container_name ] = container
            
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message="Container initialized." )
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=%s" % message
                )                

    security.declareProtected( NeoPortalModifyPortalContent, 'getNeoPortalAccessTraceKey' )        
    def getNeoPortalAccessTraceKey( self, container_name='default' ):
        """
            Return how to trace a user access
        """

        if not hasattr( aq_base( self ),  '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()

        try:
            container, access_trace_key = self._np_access_traces_containers[ container_name ]
        except KeyError:
            self.initNeoPortalAccessTraceContainer( container_name )
            container, access_trace_key = self._np_access_traces_containers[ container_name ]

        if not access_trace_key:
            return self.neoportal_access_trace_keys

        return access_trace_key
        

    security.declarePrivate( 'setNeoPortalAccessTraceKey' )     
    def setNeoPortalAccessTraceKey( self, access_trace_key, container_name='default'  ):
        """
            Return container's access trace key
        """
        if not hasattr( aq_base( self ), '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()
        
        if not self._np_access_traces_containers.has_key( container_name ):
            self.initNeoPortalAccessTraceContainer( container_name=container_name )        

        container, dummy = self._np_access_traces_containers[ container_name ]
            
        self._np_access_traces_containers[ container_name ] = ( container, access_trace_key )

    security.declarePublic( 'addNeoPortalAccessTrace' )
    def addNeoPortalAccessTrace( self, trace_value, container_name='default', date=None, ip_address=None, username=None, override_key=None, REQUEST=None ):
        """
            Add Access Trace
        """        
        if override_key is not None:
            access_trace_key = override_key
        else:        
            access_trace_key = []
            
            if 'Date' in self.getNeoPortalAccessTraceKey( container_name ):
                if date is None:
                    date = DateTime().strftime('%Y-%m-%d')
                access_trace_key.append( date )
            
            if 'IP Address' in self.getNeoPortalAccessTraceKey( container_name ):
                if ip_address is None:
                    ip_address = self._getRemoteIP()
                access_trace_key.append( ip_address )
    
            if 'Username' in self.getNeoPortalAccessTraceKey( container_name ):
                if username is None:
                    user = self.getAuthenticatedNeoPortalUser()
                    
                    if user is None or not hasattr( user, 'getUserName' ):
                        username = 'Anonymous'
                    else:
                        username = user.getUserName()    
                            
                access_trace_key.append( username )
            
            access_trace_key = self.getNeoPortalAccessTraceKeySplitter().join( access_trace_key )
        
        container = self.getNeoPortalAccessTraceContainer( container_name )
        container[ access_trace_key ] = trace_value

    security.declarePublic( 'getNeoPortalAccessTraceValue' )        
    def getNeoPortalAccessTraceValue( self, container_name='default', date=None, ip_address=None, username=None, REQUEST=None ):
        """
            Return Access Trace Value
        """        

        access_trace_key = []
        
        if 'Date' in self.getNeoPortalAccessTraceKey( container_name ):
            if date is None:
                date = DateTime().strftime('%Y-%m-%d')
            access_trace_key.append( date )
        
        if 'IP Address' in self.getNeoPortalAccessTraceKey( container_name ):
            if ip_address is None:
                ip_address = self._getRemoteIP()
            access_trace_key.append( ip_address )

        if 'Username' in self.getNeoPortalAccessTraceKey( container_name ):
            if username is None:
                user = self.getAuthenticatedNeoPortalUser()
                
                if user is None or not hasattr( user, 'getUserName' ):
                    username = 'Anonymous'
                else:
                    username = user.getUserName()    
                        
            access_trace_key.append( username )
        
        access_trace_key = self.getNeoPortalAccessTraceKeySplitter().join( access_trace_key )
        container = self.getNeoPortalAccessTraceContainer( container_name )
        if container.has_key( access_trace_key ):
            return container[ access_trace_key ]

        return None        

    security.declareProtected( NeoPortalModifyPortalContent, 'deleteNeoPortalAccessTrace' )        
    def deleteNeoPortalAccessTrace( self, container_name='default', date=None, ip_address=None, username=None, REQUEST=None ):
        """
            Delete Access Trace
        """        

        access_trace_key = []
        
        if 'Date' in self.getNeoPortalAccessTraceKey( container_name ):
            access_trace_key.append( date )
        
        if 'IP Address' in self.getNeoPortalAccessTraceKey( container_name ):
            access_trace_key.append( ip_address )

        if 'Username' in self.getNeoPortalAccessTraceKey( container_name ):
            access_trace_key.append( username )
        
        access_trace_key = self.getNeoPortalAccessTraceKeySplitter().join( access_trace_key )
        
        container = self.getNeoPortalAccessTraceContainer( container_name )
        if container.has_key( access_trace_key ):
            del container[ access_trace_key ]
        
        return 'No such access trace key: %s' % access_trace_key

    security.declarePublic( 'getNeoPortalNumAccessTraces' )
    def getNeoPortalNumAccessTraces( self, container_name='default' ):
        """
           Return the number of access traces
        """
        return len ( self.getNeoPortalAccessTraceContainer( container_name ) )

    security.declareProtected( NeoPortalModifyPortalContent, 'displayAllAccessTraceContainers' )        
    def displayNeoPortalAccessTraceContainer( self, container_name=None):
        """
            Display contents of access trace container
        """            
        
        if not hasattr( aq_base( self ), '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()
            return 'No container found.'

        if container_name is not None:            
            container, access_trace_key = self._np_access_traces_containers[ container_name ]
            if len( container ):
                return '%s has no record yet. Access Trace Key: %s' % ( container_name, self.getNeoPortalAccessTraceKey( container_name ) )
            
            message = '<p><b>%s has the following traces (key: %s):</b></p>' % ( container_name, self.getNeoPortalAccessTraceKey( container_name ) )
            
            for k in container.keys():
                message = message + '%s: %s <br />' % ( k, container[k] )

            return message
        else:
            message = '<p><b>%s has %d access trace containers.</b></p>' % ( self.getId(), len( self._np_access_traces_containers ) )

            for container_name in self._np_access_traces_containers.keys():
                container, access_trace_key = self._np_access_traces_containers[ container_name ]
                if not len( container ):
                    message = message + '<p><b>%s has no record yet. Access Trace Key: %s</b></p>' % ( container_name, self.getNeoPortalAccessTraceKey( container_name ) )
                    continue 
                message = message + '<p><b>%s has the following traces (key: %s):</b></p>' % ( container_name, self.getNeoPortalAccessTraceKey( container_name ) )
                
                for k in container.keys():
                    message = message + '%s: %s <br />' % ( k, container[k] )

            return message
        
            
InitializeClass( NeoPortalAccessTraceableObject )

