#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_BaseExternalFile.py,v 1.6 2003/01/14 19:43:05 wankyu Exp $
Description

 NeoPortal External File
 
    munge(ExternalFile+ExtFile+File). NeoPortal ExternalFile Stores file contents on the filesystem.
    
    Unlike ExternalFile or ExtFile, it inherits from OFS.Image.Image/File Class.

    NeoPortal External File instances work just like a File or Image instance with the same set of methods.
    
    NeoPortal External File is supposed to be added by a file-based product. It doesn't provide a ZMI addition feature.
 
"""
import os, sys, os.path, types, string, time

from Globals import InitializeClass, package_home
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo
from OFS.Image import Image as ZImage, File, getImageInfo
from webdav.common import rfc1123_date

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_Utils import getDTML

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from cStringIO import StringIO
import re

def copyFile( infile, outfile ):
    if type( infile ) is types.StringType: 
        instream = open( infile, 'rb' )
        close_in = 1
    else:
        instream = infile
        close_in = 0
    if type( outfile ) is types.StringType: 
        outstream = open( outfile, 'wb' )
        close_out = 1
    else:
        outstream = outfile
        close_out = 0

    blocksize = 2<<16
    block = instream.read( blocksize )
    outstream.write( block )
    while len( block )==blocksize:
        block = instream.read( blocksize )
        outstream.write( block )
    try: 
        instream.seek(0)
    except: 
        pass
    
    if close_in: 
        instream.close()
    if close_out: 
        outstream.close()


###################################################################################
# 
# Adapted from ExtFile: ExtFile's TRANSMAP messes up with Asian characters!
#
###################################################################################

FILEPATHS_FILE = package_home( globals() ) + '/data/filepath_bases.props'

if __name__ == '__main__':
    FILEPATHS_FILE = FILEPATHS_FILE.replace( '__main__', '' )

def getAllowedFileBases():
    """
        Return allowed file bases
    """    

    bases = []
    for base in [ line.strip() for line in open( FILEPATHS_FILE, 'rb' ).readlines() if not line.startswith('#') ]:
        if not base.endswith( os.sep ): 
            base = base + os.sep

        bases.append( base )

    return bases

def checkFilePath( filepath ):
    """
        Validate filepath against allowed file bases
    """
    allowed_bases = getAllowedFileBases()

    if not len( allowed_bases ): return 1

    allowed = 0
    for base in allowed_bases:
        if filepath.startswith( base ): allowed =1

    return allowed

def translateBadFileName( filename ):
    """
        Strip non-alphanumeric characters from the filename: 
            trust me it's a real headache if you allow non-ascii characters on the filesystem
    """
    org_filename = filename
    filename = re.sub( r'[^\-\_\.a-zA-Z0-9]' , '_', filename )
    
    if not org_filename == filename:
        # tag it with a timestamp
        name, ext = os.path.splitext( filename )
        filename =  name + str( int( time.time() ) ) + ext

    return filename

def addNeoPortalBaseExternalFile( self, filepath, id='', title='', description='', file='', check_permission=0, check_filepath=1, REQUEST=None ):
    """ 
        Add a NeoPortal External File
    """

    if not checkFilePath( filepath ) and check_filepath:
        raise 'NeoPortalError', "Unauthorized filepath base!"        

    if not id and hasattr( file,'filename' ): 
        id = os.path.basename( file.filename.replace( '\\', '/' ) )
        title = title or id 
        
        id = translateBadFileName( id )

    instance = NeoPortalBaseExternalFile( id=id, filepath=filepath, title=title, description=description, file=file, check_permission=check_permission, check_filepath=check_filepath )
    self._setObject( id, instance )

    if REQUEST is not None:
        return self.manage_main( self, REQUEST, update_menu=0 )

class NeoPortalBaseExternalFile( NeoPortalBaseItem, ZImage ):   
    """
    NeoPortal ExternalFile:
    
        NeoPortal External File instances behave just like normal 'File', but all data is stored 
        as an external file on the filesystem.
           
        Unlike ExternalFile or ExtFile, it inherits from OFS.Image.Image/File Class.
        NeoPortal External File instances work just like a File or Image instance with the same set of methods.
        
        Image: 
        
            * automatically viewable, thumbnails can be created and viewed by setting 
            the following in REQUEST:
                
            - view_thumbnail ( boolean ): view thumbnail, not the whole image
                
            - max_width/max_height ( int ): thumbnail size, ignored if smaller than original
                
            - keep_ratio ( boolean ): keep the ratio of thumbnail scale
                
        File:
        
            * just pass it thru
    """
    
    neoportal_external_file = 1
    
    meta_type = 'NeoPortal External File'
    icon='misc_/NeoPortalLibrary/np_external_file_icon'     

    _np_default_content_type = 'application/octet-stream'
    _np_temp_file_ext = '_temp'
    _np_allowed_file_bases = {}
    
    manage_options = (
            ({'label':'Filesystem', 'action': 'filesystem_view'},
            ) + ZImage.manage_options
        )
    
    security = ClassSecurityInfo()
    
    security.declareProtected( NeoPortalModifyPortalContent, 'filesystem_view' )
    filesystem_view = getDTML( 'dtml/NP_ExternalFileFileSystemView' )
        
    def __init__( self, id, filepath='', title='', description='', file='', check_permission=0, check_filepath=1 ):
        try:
            self.setFilePath( filepath, check_filepath=check_filepath )
            self.description = description
            self._check_permission = check_permission
    
            ZImage.__init__( self, id=id, title=title, file=file )
        except:
            raise

    ###################################################################################
    # 
    #       Public Methods - Public view methods
    #
    ###################################################################################

                    
    def __call__( self, cline=None, REQUEST=None, RESPONSE=None, **kw ):
        """
            Content-aware Public View
        """

        if REQUEST is None and hasattr( self,'REQUEST' ): 
            REQUEST = self.REQUEST
        if RESPONSE is None and REQUEST is not None:
            RESPONSE = REQUEST.RESPONSE

        view_thumbnail = 0
        if REQUEST is not None:
            view_thumbnail = int( REQUEST.get( 'view_thumbnail', 0 ) )
            max_width = int( REQUEST.get( 'max_width', 0 ) )
            max_height = int( REQUEST.get( 'max_height', 0 ) )
            keep_ratio = int( REQUEST.get( 'keep_ratio', 1 ) )

        if not max_width: max_width = 10
        if not max_height: max_height = 10
       
        if view_thumbnail and self.isImage():
            if keep_ratio:
                width = float( max_width )/float( self.width )
                height = float( max_height )/float( self.height )
                scale = min( [ width, height ] )
                                
                return self.tag( width=self.width, height=self.height, scale=scale, border=1 )
        else:
            return self.index_html( REQUEST, RESPONSE )            
            
    security.declarePublic( 'index_html' )
    def index_html( self, REQUEST=None, RESPONSE=None ):
        """ 
            Adapted from ExtFile's index_html / OFS.Image.File's index_html
        """

        if RESPONSE is not None:
            RESPONSE.setHeader( 'Content-Disposition', 'attachment; filename=%s' % self.getFileName() )

        if not self._isFileAccessible( REQUEST ):
            message = self.getNeoPortalMessage( message="You don't have permission to view or download this file." )                        
            raise Unauthorized, message

        if REQUEST is None and hasattr( self,'REQUEST' ): 
            REQUEST = self.REQUEST

        if RESPONSE is None and REQUEST is not None:
            RESPONSE = REQUEST.RESPONSE            

        if REQUEST is not None:
            header = REQUEST.get_header( 'If-Modified-Since', None )
            if header is not None:
                header = ';'.split(header)[0]
                try:    mod_since = long(DateTime(header).timeTime())
                except: mod_since = None
                if mod_since is not None:
                    if self._p_mtime:
                        last_mod = long( self._p_mtime)
                    else:
                        last_mod = long(0)
                    if last_mod > 0 and last_mod < mod_since:
                        REQUEST.RESPONSE.setStatus(304)
                        return ''

        filename = self.getFilePath()
        content_type = self.getContentType()
        
        if not os.path.isfile( filename ): 
            self._restore()
            filename = self.getFilePath()
            if not os.path.isfile( filename ): 
                return "File not found: %s", filename
        else:
            size = os.stat(filename)[6] # filesize
        
        # data passthru
        data = open(filename, 'rb')
        try:
            if REQUEST is not None:
                last_mod = rfc1123_date( self._p_mtime)
                RESPONSE.setHeader('Last-Modified', last_mod)
                RESPONSE.setHeader('Content-Type', content_type)
                RESPONSE.setHeader('Content-Length', size)
                blocksize = 2<<16
                while 1:
                    buffer = data.read(blocksize)
                    RESPONSE.write(buffer)
                    if len(buffer) < blocksize:
                        break
                return ''
            else:
                return data.read()
        finally:
            data.close()   

    security.declarePublic( 'thumb' )
    def thumb( self, REQUEST=None, RESPONSE=None ):
        """
            Return a thumbnail image
        """

        return self.__call__( REQUEST=REQUEST, RESPONSE=RESPONSE )

    def __str__( self ):
        """
           Return String Representation of External File: Image/File
        """
        
        return self.getId()
        
    security.declareProtected( NeoPortalModifyPortalContent, 'getAllowedFileBases' )
    def getAllowedFileBases( self ):
        """
            Return allowed file bases
        """

        return getAllowedFileBases()

    ###################################################################################
    # 
    #       Public Methods - Content probing methods
    #
    ###################################################################################


    security.declarePublic( 'get_size' )
    def get_size( self ):
        """
            Return File Size
        """
        return self.__len__()

    def __len__( self ): 
        """
            Return File Size
        """
        if os.path.exists( self.getFilePath() ):
            self.size = os.stat( self.getFilePath() )[6]
            return self.size
        else: 
            return 0
        
    security.declarePublic( 'getFilePath' )
    def getFilePath( self ):
        """
            Return File Path
        """     
        
        return self.filepath
    
    security.declarePublic( 'getFileName' )
    def getFileName( self ):
        """
            Return File Name
        """     
        return os.path.basename( self.filepath )
    
    security.declarePublic( 'getFileSize' )
    def getFileSize( self ):
        """
            Return File Size
        """     
        try:
            filesize = os.stat( self.getFilePath() )[6]
        except:
            filesize = 1
        return filesize

    security.declarePublic( 'getFileSizeInKB' )
    def getFileSizeInKB( self ):
        """
            Return File Size in KB
        """     
        return round( int( self.getFileSize()/1024 ) )

    security.declarePublic( 'getFileSizeInMB' )
    def getFileSizeInMB( self ):
        """
            Return File Size in MB
        """     
        return round(int( self.getFileSize()/1047576 ) )
    
    security.declarePublic( 'getFileModTime' )
    def getFileModTime( self ):
        """
            Return File Modification Time
        """     
        return os.path.getmtime( self.getFilePath() )

    security.declarePublic( 'getModTime' )
    def getModTime( self ):
        """
            Return Object Modification Time
        """             
        file_mtime = self.getFileModTime()
        if self._p_mtime > file_mtime:
            return self._p_mtime
        else:
            return file_mtime

    ###################################################################################
    # 
    #       Public Methods - Image specific methods
    #
    ###################################################################################


    security.declarePublic( 'isImage' )
    def isImage( self ):
        """
            Tell if file content is image
        """
        
        if hasattr( self, '_is_image' ):
            return self._is_image        
        
        try:
            data = StringIO()
            copyFile( self.getFilePath(), data )
    
            content_type, width, height = getImageInfo( data.getvalue() )

            is_image = 0
            if content_type.split('/')[0].lower() == 'image':
                is_image = 1
            return is_image
        except:
            return 0
       
    security.declarePublic( 'getImageSize' )
    def getImageSize( self ):
        """
            Return ImageSize
        """        
        return self.width, self.height     

    security.declarePublic( 'checkFilePath' )
    def checkFilePath( self, filepath ):    
        """
            Validate filepath against allowed file bases
        """

        return checkFilePath( filepath ) 
    

    ###################################################################################
    # 
    #       Private Methods - CMF DublinCore compatible methods
    #
    ###################################################################################

    security.declareProtected( NeoPortalModifyPortalContent, 'setTitle' )
    def setTitle( self, title ):
        """
            Set title
        """
        
        self.title = title

    security.declareProtected( NeoPortalModifyPortalContent, 'setDescription' )
    def setDescription( self, description ):
        """
            Return description: for compatibility with CMF DublinCore
        """ 
        
        self.description = description
        

    ###################################################################################
    # 
    #       Private Methods - Content creating/updating methods
    #
    ###################################################################################

    security.declareProtected( NeoPortalView, 'fileExists' )
    def fileExists( self, filepath ):
        """
            Tell if file exists: return false when the instance is attemting to 
            overwrite its own file.
        """     
        
        if os.path.isfile( filepath ) and self.getFilePath() != filepath:
            return 1

        return 0

    security.declareProtected( NeoPortalModifyPortalContent, 'setFilePath' )
    def setFilePath( self, filepath, check_filepath=1 ):
        """
            Set File Path
        """     
        
        if os.path.isdir( filepath ):
            message = self.getNeoPortalMessage( message="Cannot set filepath to a directory: %s", tup=( filepath, ) )                        
            raise 'NeoPortalError', message

        if check_filepath and not self.checkFilePath ( filepath ):
            message = self.getNeoPortalMessage( message="You don't have permissions to use the path: %s", tup=( filepath, ) )                        
            raise 'NeoPortalError', message

        dirname = os.path.dirname( filepath )
        if not dirname.endswith( os.sep ): dirname = dirname + os.sep
        filename = os.path.basename( filepath.replace( '\\', '/' ) )
        filename = translateBadFileName( filename )
        
        self.filepath = dirname + filename
        
    security.declareProtected( NeoPortalModifyPortalContent, 'manage_upload' )
    def manage_upload( self,file='',archive_filename='', REQUEST=None ):
        """
            Change filename if not the same
        """
        
        # filename has changed, delete the old file
        if self.getFileName() != archive_filename:
            self._delete_file()

        return ZImage.manage_upload( self, file=file, REQUEST=REQUEST )   
            
    security.declareProtected( NeoPortalModifyPortalContent, 'manage_delete' )
    def manage_delete( self ):
        """
            Delete NeoPortal ExternalFile
        """
        self._delete_file()
        self.aq_inner.aq_parent._delObject( self.getId() )        
        
    security.declareProtected( NeoPortalModifyPortalContent, 'update_data' )
    def update_data( self, data, content_type=None, size=None ):
        if content_type is not None:
            self.content_type = content_type
        if size is not None:
            self.size = size

        ct, width, height = getImageInfo( data )

        if width >= 0 and height >= 0:
            self.width = width
            self.height = height

        self.ZCacheable_invalidate()              
    
    security.declarePrivate( '_read_data' )
    def _read_data( self, file ):
        """
            Read file and save it
        """
        copyFile( file, self.getFilePath() )        

        data = StringIO()
        
        copyFile( self.getFilePath(), data )
        
        self.size = self.get_size()
        
        return data.getvalue(), self.size
    
    security.declarePrivate( '_delete_file' )
    def _delete_file( self ):
        """
            Unlink file on filesystem
        """
        try:
            os.unlink( self.getFilePath() )
        except:
            pass

    security.declarePrivate( '_make_temp_file' )
    def _make_temp_file( self ):
        """
            Create a temporary backup file
        """
        try:
            os.rename( self.getFilePath(), self._getTempFileName() )
        except:
            pass

    security.declarePrivate( '_restore' )
    def _restore( self ):
        """ 
            Restore filename after undo or copy-paste 
        """
        temp_filename = self._getTempFileName()
        
        if not os.path.isfile( self.getFilePath() ) and os.path.isfile( temp_filename ):
            try:
                os.rename( temp_filename, self.getFilePath() )            
            except:
                pass

    security.declarePrivate( '_getTempFileName' )
    def _getTempFileName( self ):
        """
            Return temprary file extension
        """        
        return self.getFilePath()+ '.' + self._np_temp_file_ext

    security.declarePrivate( '_getNewFilePath' )
    def _getNewFilePath( self ):
        """
            Return a new file path for copy/paste operations (tagged with a timestamp)            
        """             

        new_filepath =  os.path.splitext( self.getFilePath())[0] + '_' + str( int( time.time() ) ) + os.path.splitext( self.getFilePath() )[1]
        while 1:
            # rarely happens, but it happens with fast reloads
            if os.path.isfile( new_filepath ):
                new_filepath =  os.path.splitext( self.getFilePath() )[0] + '_' + str( int( time.time() ) ) + os.path.splitext( self.getFilePath() )[1]
            else:
                break
        
        return new_filepath        

    security.declarePrivate( '_isFileAccessible' )
    def _isFileAccessible( self, REQUEST ):
        """
            Adapted from ExtFile's _access_permitted() method
        """
        
        if not hasattr( self, '_check_permission' ) or not self._check_permission:
            return 1
        
        user = getAuthenticatedNeoPortalUser( REQUEST )
        if ( user is None ) or not hasattr( user, 'has_permission') or not user.has_permission( NeoPortalView, self ):
            return 0
        else: 
            return 1
            
    security.declarePublic( 'checkFilePath')
    def checkFilePath( self, filepath ):
        """
            Validate the given path against allowed file bases
        """

        return checkFilePath( filepath )            
                        
    ###################################################################################
    # 
    #       Manage Methods - automagically called methods
    #
    ###################################################################################

    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete( self, item, container ):
        """
            Put the file in a temporary state
        """     
        
        self._make_temp_file()
        return NeoPortalBaseExternalFile.inheritedAttribute("manage_beforeDelete")( self, item, container )

    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd( self, item, container ):
        """
            Tell if self is being cut and pasted
        """
         
        self._restore()
        return NeoPortalBaseExternalFile.inheritedAttribute("manage_afterAdd")( self, item, container )

    security.declarePrivate( 'manage_afterClone' )
    def manage_afterClone( self, item, new_fn=None ):
        """ 
            Copy and pasted: NeoPortal External File is a 'hard link'. A copy of the file should also be created.
        """

        copied_file = self._getNewFilePath()
        
        copyFile( self.getFilePath(), copied_file )
        
        self.filepath = copied_file

        return NeoPortalBaseExternalFile.inheritedAttribute("manage_afterClone")( self, item )
    
InitializeClass( NeoPortalBaseExternalFile )