#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_Catalog.py,v 1.4 2003/04/01 14:34:46 wankyu Exp $
Description

    NeoPortal Catalog - modeled after CMF's CatalogTool    
"""

import os
from Globals import InitializeClass
from Products.ZCatalog.ZCatalog import ZCatalog
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent, aq_base, aq_inner

try:
    from Products.TextIndexNG import TextIndexNG
    TEXTINDEXNG_INSTALLED=1
except:
    TEXTINDEXNG_INSTALLED=0

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalCatalog ( ZCatalog ):
    """
        NeoPortal Catalog Tool
    """
    meta_type = 'NeoPortal Catalog'

    _np_catalog_indexes = ( ('Title', 'TextIndexNG')
                           , ('meta_type', 'FieldIndex')
                           , ('CreationDate', 'FieldIndex')        
                           , ('Creator', 'FieldIndex')
                           , ('getNeoPortalContentSearchText', 'TextIndexNG')
                           , ('getInlineCommentsSearchText', 'TextIndexNG')
                           , ('getInlineCommentCreators', 'TextIndexNG')
                           , ('getNeoPortalReadCount', 'FieldIndex')
                           , ('getNeoPortalNumContentRatings', 'FieldIndex')
                           )
    _np_catalog_schema = ( 'Title'
                         , 'Creator'
                         )
                         
    _np_cataloggable_meta_types = []
                         
    _np_textindex_ng_options = { 'default_encoding'     :   'UTF-8'
                               , 'useSplitter'          :   'TXNGSplitter'
                               , 'splitterMaxLen'       :   64
                               , 'splitterSingleChars'  :   0
                               , 'splitterSeparators'   :   ''
                               , 'splitterCasefolding'  :   1
                               , 'useStemmer'           :   None
                               , 'lexicon'              :   None
                               , 'useSimilarity'        :   None
                               , 'nearDistance'         :   5
                               , 'useStopwords'         :   None
                               , 'useNormalizer'        :   None
                               , 'useConverters'        :   0
                               , 'useCL'                :   0
                               , 'useParser'            :   'PyQueryParser'
                            }

    security = ClassSecurityInfo()

    manage_options = ZCatalog.manage_options
    
    def __init__( self ):
        
        ZCatalog.__init__( self, self.getId() )

        self._initIndexes()

    security.declarePublic( 'getNeoPortalElementContainerCataloggableMetaTypes' )
    def getNeoPortalElementContainerCataloggableMetaTypes( self ):
        """
            Return cataloggable meta types
        """
    
        return self._np_cataloggable_meta_types

    security.declarePublic( 'enumerateIndexes' )
    def enumerateIndexes( self ):
        """
            Return Indexes
        """

    security.declarePublic( 'enumerateIndexes' )
    def enumerateIndexes( self ):
        """
            Return a list of ( index_name, type ) pairs for the initial
            index set.
        """ 
        
        return self._np_catalog_indexes        
    
    security.declarePublic( 'enumerateColumns' )
    def enumerateColumns( self ):
        """
            Return schema
        """
        return self._np_catalog_schema
        
    security.declarePrivate( '_initIndexes' )
    def _initIndexes( self ):

        self._catalog.indexes.clear()
        for index_name, index_type in self.enumerateIndexes():
            extra = None
            if index_type == 'TextIndexNG':
                if TEXTINDEXNG_INSTALLED:
                    extra = self._np_textindex_ng_options
                else:
                    index_type = 'TextIndex'
                
            self.addIndex( index_name, index_type, extra=extra )

        # Cached metadata
        self._catalog.names = ()
        self._catalog.schema.clear()
        for column_name in self.enumerateColumns():
            self.addColumn( column_name )

    security.declarePrivate( '_getObjectURL' )
    def _getObjectURL( self, ob ):
        """
            Return Object URL
        """
        return '/'.join( ob.getPhysicalPath() )

    security.declarePrivate( 'indexObject' )
    def indexObject( self, object ):
        """
            Add to the catalog
        """

        url = self._getObjectURL( object )
        self.catalog_object( object, url )

    security.declarePrivate( 'unindexObject' )
    def unindexObject( self, object ):
        """
            Remove from the catalog
        """

        url = self._getObjectURL( object )
        self.uncatalog_object( url )

    security.declarePrivate( 'reindexObject' )
    def reindexObject( self, object, idxs=[] ):
        """
            Update the catalog
            idx = list of indexes to update (optional)
        """
        url = self._getObjectURL( object )

        if idxs != []:
            # Filter out invalid indexes.
            valid_indexes = self._catalog.indexes.keys()
            idxs = [ i for i in idxs if i in valid_indexes ]
        self.catalog_object( object, url, idxs=idxs )        

InitializeClass( NeoPortalCatalog )