#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_Content.py,v 1.15 2003/04/01 14:34:46 wankyu Exp $
Description

    NeoPortal Content gathers all the mix-ins a NeoPortal Content Object requires
    
    When reindexing or recooking content is required, the '_updateNeoPortalContent' method is called.
 
    All NeoPortal Content Types should implement/call this '_updateNeoPortalContent' method when content gets changed.
    
    A series of 'renderer' methods can be defined thru which raw content can be pushed.
              
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from Products.NeoPortalLibrary.NP_InlineCommentableContent import NeoPortalInlineCommentableContent
from Products.NeoPortalLibrary.NP_RatableContent import NeoPortalRatableContent
from Products.NeoPortalLibrary.NP_IconizableContent import NeoPortalIconizableContent
from Products.NeoPortalLibrary.NP_UpgradableObject import NeoPortalUpgradableObject
from Products.NeoPortalLibrary.NP_AccessLoggableObject import NeoPortalAccessLoggableObject
from Products.NeoPortalLibrary.NP_TranslatableContent import NeoPortalTranslatableContent
from Products.NeoPortalLibrary.NP_QuotaAwareObject import NeoPortalQuotaAwareObject
from Products.NeoPortalLibrary.NP_Utils import getDTML, removeTags

import re

class NeoPortalContent( NeoPortalUpgradableObject
                      , NeoPortalInlineCommentableContent
                      , NeoPortalIconizableContent
                      , NeoPortalRatableContent
                      , NeoPortalAccessLoggableObject
                      , NeoPortalTranslatableContent                      
                      , NeoPortalQuotaAwareObject
                      , NeoPortalBaseItem ):
    """
        The sum of all NeoPortal Content-specific mix-in classes
    """
    
    neoportal_content = 1
    neoportal_advanced_content = 0
    neoportal_discussable_content = 1
    
    allow_discussion = 0 # CMF? we're not using Portal Discussion Tool.
    
    _np_renderers = ['iconizeNeoPortalContent', '_cookContent', '_insertInlineCommentFormMarker', '_renderInlineComments', '_renderEmbeddedObjects']
    
    security = ClassSecurityInfo()    
    

    security.declareProtected( NeoPortalManageProperties, 'manage_editNeoPortalContentOptions' )
    def manage_editNeoPortalContentOptions( self
                                        , allow_inline_comments=0
                                        , max_inline_comment_size=1000                                        
                                        , allow_content_rating=0
                                        , max_content_rating_value=10
                                        , min_content_rating_value=1
                                        , content_rating_access_trace_key=[]
                                        , allow_content_iconization=0
                                        , allow_content_discussion=0
                                        , authors=[]
                                        , enable_access_logging=0
                                        , access_log_access_trace_key=[]
                                        , user_response_access_trace_key=[]
                                        , purge_access_log=0
                                        , access_log_purge_days=0
                                        , initialize_content_ratings=0
                                        , enable_spam_blocking=0
                                        , spam_blocking_interval=10
                                        , enable_quota=0
                                        , size_quota=0
                                        , number_quota=0
                                        , REQUEST=None ):
        """ 
            Edit NeoPortal Content Options    
        """
        
        if hasattr( aq_base( self ), 'neoportal_element' ):
            if REQUEST is not None:
                message = self.getNeoPortalMessage( message="Elements inherit options from their containers." )
                return REQUEST['RESPONSE'].redirect(
                    self.absolute_url()
                    + "?portal_status_message=%s" % message
                    )
                    
            return

        if max_content_rating_value and min_content_rating_value and ( int( max_content_rating_value ) <= int( min_content_rating_value ) ):
            if getattr( self, 'np_ratable_content', 0):
                if REQUEST is not None:
                    message = self.getNeoPortalMessage( message="Max content rating value should be bigger than the min value." )
                    return REQUEST['RESPONSE'].redirect(
                        self.absolute_url()
                        + "?portal_status_message=%s" % message
                        )
                return
            
        # prevent users from accidentally setting properties to false
        
        if allow_inline_comments or ( not allow_inline_comments and hasattr( self, 'np_inline_comments_allowed' ) ):
            self.np_inline_comments_allowed = allow_inline_comments
            
        if allow_content_rating or ( not allow_content_rating and hasattr( self, 'np_content_rating_allowed') ):
            self.np_content_rating_allowed = allow_content_rating

        if allow_content_iconization or ( not allow_content_iconization and hasattr( self, 'np_content_iconization_allowed') ):
            self.np_content_iconization_allowed = allow_content_iconization

        if allow_content_discussion or ( not allow_content_discussion and hasattr( self, 'np_content_discussion_allowed') ):
            self.np_content_discussion_allowed = allow_content_discussion

        if enable_access_logging or ( not enable_access_logging and hasattr( self, 'np_access_logging_enabled') ):
            self.np_access_logging_enabled = enable_access_logging            

        if enable_spam_blocking or ( not enable_spam_blocking and hasattr( self, 'np_spam_blocking_enabled') ):
            self.np_spam_blocking_enabled = enable_spam_blocking 

        if spam_blocking_interval or ( not spam_blocking_interval and hasattr( self, 'np_spam_blocking_interval') ):
            self.np_spam_blocking_interval = spam_blocking_interval 

        if max_inline_comment_size or ( not max_inline_comment_size and hasattr( self, 'np_max_inline_comment_size') ):
            self.np_max_inline_comment_size = max_inline_comment_size 

        if max_content_rating_value or ( not max_content_rating_value and hasattr( self, 'np_max_content_rating_value') ):
            if int( max_content_rating_value ) <= self._np_content_rating_value_top_limit:
                self.np_max_content_rating_value = max_content_rating_value 

        if access_log_access_trace_key and 'Date' in access_log_access_trace_key and access_log_access_trace_key != self.getNeoPortalAccessTraceKey( container_name=self._np_access_log_container_name ):
            self.setNeoPortalAccessTraceKey( access_trace_key=access_log_access_trace_key, container_name=self._np_access_log_container_name )

        if content_rating_access_trace_key and ( 'IP Address' in content_rating_access_trace_key  or 'Username' in content_rating_access_trace_key )and content_rating_access_trace_key != self.getNeoPortalAccessTraceKey( container_name=self._np_content_rating_container_name ):
            self.setNeoPortalAccessTraceKey( access_trace_key=content_rating_access_trace_key, container_name=self._np_content_rating_container_name )

        if min_content_rating_value or ( not min_content_rating_value and hasattr( self, 'np_min_content_rating_value') ):
            if int( min_content_rating_value ) >= self._np_content_rating_value_bottom_limit:
                self.np_min_content_rating_value = min_content_rating_value 

        if purge_access_log and hasattr( self, 'np_access_logging_enabled' ):
            self.purgeNeoPortalAccessLog( purge_days = access_log_purge_days )

        if initialize_content_ratings and hasattr( self, 'np_content_rating_allowed' ):
            self.initNeoPortalContentRatings()
        
        if authors or ( not authors and hasattr( self, 'authors' ) ):
            self.authors=authors

        # won't work without CMF
        try:
            if int( allow_content_discussion ) and not hasattr( self, 'np_neoboard' ):
                neoboard = self.neoportal_discussion.getDiscussionFor( self )
                
            if not int( allow_content_discussion ) and hasattr( self, 'np_neoboard' ):
                self.np_neoboard.setNeoBoardEnabled( mode=0 )
        except Exception, e:
            self.np_log(message="Content Discussion not working: %s" % e, summary='NP_Content', severity='INFO')
            pass

        if getattr( aq_base( self ), 'neoportal_quota_container', 0 ):
            self.neoportal_quota_enabled = enable_quota
            self._neoportal_container_size_quota = int( size_quota )
            self._neoportal_container_number_quota = int( number_quota )
                
        self._updateNeoPortalContent()
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( REQUEST=REQUEST, message="Changes applied." )
            return REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + '?portal_status_message=%s' % message
                )

    security.declareProtected( NeoPortalModifyPortalContent, 'getNeoPortalContentSearchText' )
    def getNeoPortalContentSearchText(self):
        """
            Return Search Text
        """
        
        return removeTags( ' '.join( ( self.Title(), self.Description(), self.cooked_text , ) ) )
        
        
    PrincipiaSearchSource = getNeoPortalContentSearchText        

    security.declarePublic( 'Description' )
    def Description( self, **kw ):
        """
            Return description after remove any embedding markers
        """
        
        return self._stripAllEmbedMarkers( self.description )
        
    security.declarePublic( 'setDescription' )
    def setDescription( self, description ):
        """
            Set Description
        """
        
        self.description = description
        self._updateNeoPortalContentDescription()

    security.declarePrivate( '_updateNeoPortalContentDescription' )
    def _updateNeoPortalContentDescription( self, **kw ):
        """
            Update cooked description
        """     
        renderers = [ 'iconizeNeoPortalContent', '_cookContent', '_renderEmbeddedObjects' ]
        self.np_cooked_description = self._renderNeoPortalContent( content = self.description, renderers=renderers )    
    
    security.declarePublic( 'getRenderedNeoPortalContentDescription' )
    def getRenderedNeoPortalContentDescription( self, force=0, **kw ):
        """
            Return cooked ontent
        """     

        if not hasattr( aq_base( self ), 'np_cooked_description' ) or force:
            self._updateNeoPortalContentDescription()
        
        return self.np_cooked_description
        
    security.declarePublic( 'Language' )
    def getNeoPortalContentLanguage( self ):
        """
            Return Language
        """
        
        if hasattr( self, 'language' ) and self.language:
            return self.language
        try:
            return self.portal_properties.site_properties.default_language
        except:
            return 'en'
            
    security.declareProtected( NeoPortalManagePortal, 'selfUpgradeNeoPortalContent' )
    def selfUpgradeNeoPortalContent( self, force_upgrade=0):
        """
            Upgrade Self: force_upgrade does what its name implies
        """
        
        if self.isNeoPortalObjectUpgradeNeeded():
            self.notifyNeoPortalObjectUpgradeComplete()
        else:
            pass
    
            
InitializeClass( NeoPortalContent )

from OFS.DTMLMethod import DTMLMethod

class NeoPortalAdvancedContent( NeoPortalContent ):
    """
        The sum of all Advanced NeoPortal Content-specific mix-in classes
    """
    
    neoportal_advanced_content = 1
    
    security = ClassSecurityInfo()    
    
    security.declarePublic( 'dtmlDescription' )
    def dtmlDescription( self, REQUEST=None ):
        """
            Return description after rendering DTML
        """
        
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        dtml_text = DTMLMethod( self.getRenderedNeoPortalContentDescription() )
        dtml_text = apply(dtml_text.__call__, ( self, REQUEST ) )

        return dtml_text
            
InitializeClass( NeoPortalAdvancedContent )
