#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_Element.py,v 1.4 2003/03/30 22:43:33 wankyu Exp $
Description

    NeoPortal Element is a content type that's supposed to be added to a NeoPortal Element Container object.
 
"""

from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_Utils import natSortElements

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalElement( NeoPortalBaseItem ):
    """ 
        NeoPortal Element Mix-in:
            
            NeoPortal Elements are a series of objects that can be numbered into container objects such as NeoPortal Article
    """

    neoportal_element = 1
    neoportal_element_prefix = 'element_' # should end with '_' and should not contain any more '_'

    security = ClassSecurityInfo()    
    
    security.declarePublic( 'getPrevNeoPortalElement' )
    def getPrevNeoPortalElement( self ):
        """
            Return previous element or none if not present
        """
        
        #
        # we just can't return previous element using construct 'try/except' since element renumbering feature might be enabled 
        #
        
       
        elements= [ brain for brain in self.getNeoPortalElements( e_type=self.meta_type ) ]
        if not len( elements ): return None

        id_nums = [ int( brain.getPath().split('/').pop().split( '_' )[1] ) for brain in elements ]
                
        # elements[idx  - 1] Return the last element of the list without raising any exception!
        
        try:
            idx = id_nums.index( self.getNeoPortalElementNumber() )
            return elements[idx-1].getObject()
        except:
            return None
        

    security.declarePublic( 'getNextNeoPortalElement' )
    def getNextNeoPortalElement( self ):
        """
            Return next element or none if not present
        """

        #
        # we just can't return next element using construct 'try/except' since element renumbering feature might be enabled 
        #

                
        elements= [ brain for brain in self.getNeoPortalElements( e_type=self.meta_type ) ]
        if not len( elements ): return None

        id_nums = [ int( brain.getPath().split('/').pop().split( '_' )[1] ) for brain in elements ]
                        
        try:
            return elements[ id_nums.index( self.getNeoPortalElementNumber() ) + 1 ].getObject()
        except:
            return None
    
    security.declarePublic( 'getNeoPortalElementPrefix' )
    def getNeoPortalElementPrefix( self ):
        """
            Return the Element Prefix
        """
                
        return self.getId().split('_')[0]
    
    security.declarePublic( 'getNeoPortalElementNumber' )
    def getNeoPortalElementNumber( self ):
        """
            Return the Element Number
        """
                
        return int( self.getId().split('_')[1] )

    security.declarePrivate( '_notifyOfCopyTo' )
    def _notifyOfCopyTo( self, container, op=0 ):
        """
            We can't paste NeoPortal Elements into non-Element container objects
            op 0 : copy
            op 1 : move
        """
            
        if op: 
            op_str = 'move'
        else: 
            op_str = 'copy'
        
        if not getattr( container, 'neoportal_element_container', 0 ): 
            self.np_log( message="attemp to %s %s(%s) into non-container %s(%s)" % ( op_str, self.getId(), self.meta_type, container.getId(), container.meta_type ), summary='NP_Element', severity='INFO')
            message = self.getNeoPortalMessage( message="You can't %s this object <em>%s</em> into non-NeoPortal Container object <em>%s</em>!", tup=( op_str, self.getId(), container.getId() ) )                    
            raise 'NeoPortalError', message

    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd( self, item, container):
        """
            Make sure it gets cataloged
        """             
        
        item.getNeoPortalElementContainer().addToNeoPortalElementContainerCatalog( item )

    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete( self, item, container):
        """
            Make sure it gets uncataloged
        """             
        
        item.getNeoPortalElementContainer().deleteFromNeoPortalElementContainerCatalog( item )


class NeoPortalAdvancedElement( NeoPortalElement ):
    """ 
        NeoPortal Advanced Element Mix-in:
    """

    neoportal_advanced_element = 1

    security = ClassSecurityInfo()    
    
    security.declarePrivate( '_notifyOfCopyTo' )
    def _notifyOfCopyTo( self, container, op=0 ):
        """
            We can't paste NeoPortal Elements into non-Element container objects
            op 0 : copy
            op 1 : move
        """
            
        if op: 
            op_str = 'move'
        else: 
            op_str = 'copy'
        
        if not getattr( container, 'neoportal_advanced_element_container', 0 ): 
            self.np_log( message="attemp to %s %s(%s) into non-container %s(%s)" % ( op_str, self.getId(), self.meta_type, container.getId(), container.meta_type ), summary='NP_Element', severity='INFO')
            message = self.getNeoPortalMessage( message="You can't %s this object <em>%s</em> into non-NeoPortal Advanced Container object <em>%s</em>!", tup=( op_str, self.getId(), container.getId() ) )                    
            raise 'NeoPortalError', message