#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_ElementContainer.py,v 1.17 2003/04/02 07:57:15 wankyu Exp $
Description
    
    NeoPortal Element Container is a folderish object that can hold NeoPortal Elements

    NeoPortal Element Containers can be nested, in which case only the top-most container should have 
    the '_np_top_most_element_container' Class property set to true.
"""

from Globals import InitializeClass, MessageDialog
from OFS.DTMLMethod import DTMLMethod
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner

from Products.ZCatalog.Catalog import Catalog as BaseCatalog
from Products.ZCatalog.CatalogBrains import AbstractCatalogBrain
#
# Try importing BTreeFolder first: BTreeFolder is always prefered
#

try:
    from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2 as BaseFolder
    BTREEFOLDER_INSTALLED = 1
except ImportError:
    from OFS.Folder import Folder as BaseFolder
    BTREEFOLDER_INSTALLED = 0

from Products.NeoPortalLibrary.NP_Catalog import NeoPortalCatalog


######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################
import re, sys

class NeoPortalElementContainerCatalog( BaseCatalog ):
    """
        NeoPortal Element Container Catalog
    """
    
    def useBrains( self, brains ):
        """ 
            Fix up getObject
        """

        class npbrains( AbstractCatalogBrain, brains ):
            def getPath(self):
                """
                    Return Container ID + UID
                """
                return self.aq_parent.getpath( self.data_record_id_ )
        
            def getURL(self, relative=0):
                """Try to generate a URL for this record"""
                try:
                    return self.REQUEST.physicalPathToURL( self.getPath(), relative )
                except:
                    return self.getPath()
        
            def getObject(self, REQUEST=None):
                """Try to return the object for this record"""
                try:
                    obj = self.aq_parent.unrestrictedTraverse( '/'.join( self.aq_parent.getPhysicalPath() ) + self.aq_parent.getpath( self.data_record_id_ ) )
                    if not obj:
                        if REQUEST is None:
                            REQUEST = self.REQUEST
                        obj = self.aq_parent.resolve_url( '/'.join( self.aq_parent.getPhysicalPath() ) + self.aq_parent.getpath( self.data_record_id_ ), REQUEST )
                    return obj
                except:
                    import zLOG
                    zLOG.LOG('NeoPortalCatalogBrains', zLOG.INFO, 'getObject raised an error',
                             error=sys.exc_info())
                    pass
            
        
            def getRID(self):
                """Return the record ID for this object."""
                return self.data_record_id_
                
        scopy = self.schema.copy()

        scopy['data_record_id_']=len(self.schema.keys())
        scopy['data_record_score_']=len(self.schema.keys())+1
        scopy['data_record_normalized_score_']=len(self.schema.keys())+2

        npbrains.__record_schema__ = scopy

        self._v_brains = brains
        self._v_result_class = npbrains
    
class NeoPortalElementContainer( BaseFolder, NeoPortalCatalog ):
    """ 
        NeoPortal Element Container
    """

    neoportal_element_container = 1
    _np_top_most_element_container = 1   # containers may get nested; we need a flag that claims self is the top-most containter
    
    _np_element_renumbering_allowed = 1
    
    manage_options = ( BaseFolder.manage_options + NeoPortalCatalog.manage_options )
    
    manage_options=(
            {'label': 'Contents',    'action':'manage_main'},
            {'label': 'View',        'action':'index_html'},
            {'label': 'Catalog',     'action': 'manage_catalogView',    'help':('ZCatalog','ZCatalog_Cataloged-Objects.stx')},
            {'label': 'Properties',  'action': 'manage_propertiesForm', 'help': ('OFSP','Properties.stx')},
            {'label': 'Indexes',     'action': 'manage_catalogIndexes', 'help': ('ZCatalog','ZCatalog_Indexes.stx')},
            {'label': 'Metadata',    'action': 'manage_catalogSchema',  'help':('ZCatalog','ZCatalog_MetaData-Table.stx')},
            {'label': 'Find Objects','action': 'manage_catalogFind',    'help':('ZCatalog','ZCatalog_Find-Items-to-ZCatalog.stx')},
            {'label': 'Advanced',    'action': 'manage_catalogAdvanced','help':('ZCatalog','ZCatalog_Advanced.stx')},
            {'label': 'Undo',        'action': 'manage_UndoForm',       'help': ('OFSP','Undo.stx')},
            {'label': 'Security',    'action': 'manage_access',         'help': ('OFSP','Security.stx')},
            {'label': 'Ownership',   'action': 'manage_owner',          'help': ('OFSP','Ownership.stx'),}
        )        

    _np_catalog_indexes  = ( ('Title', 'TextIndexNG')
                           , ('meta_type', 'FieldIndex')
                           , ('CreationDate', 'FieldIndex')
                           , ('Creator', 'FieldIndex')
                           , ('getNeoPortalContentSearchText', 'TextIndexNG')
                           , ('getInlineCommentsSearchText', 'TextIndexNG')
                           , ('getInlineCommentCreators', 'TextIndexNG')
                           , ('getNeoPortalReadCount', 'FieldIndex')
                           , ('getNeoPortalNumContentRatings', 'FieldIndex')
                           , ('getNeoPortalElementNumber', 'FieldIndex')
                           )

    _np_cataloggable_meta_types = []
    
    _np_catalog_threshold = 1000
    
    security = ClassSecurityInfo()
    
    def __init__( self, id ):
        if BTREEFOLDER_INSTALLED:
            BaseFolder.__init__( self, id=id )
        else:
            self.id = id

        self._initNeoPortalElementContainerCatalog()

    security.declarePublic( 'getNeoPortalElementContainerCataloggableMetaTypes' )
    def getNeoPortalElementContainerCataloggableMetaTypes( self ):
        """
            Return cataloggable meta types
        """
    
        return self._np_cataloggable_meta_types        
    
    security.declarePrivate( '_initNeoPortalElementContainerCatalog' )
    def _initNeoPortalElementContainerCatalog( self ):
        """
            Initialize Internal Catalog
        """
        
        self.vocabulary = None
        self.vocab_id = ''
        self._v_total = 0
        self._catalog = NeoPortalElementContainerCatalog()
        self.threshold = self._np_catalog_threshold
        self._initIndexes()
        
        import time
        elapse = time.time()
        c_elapse = time.clock()
        catalog = self.getNeoPortalElementContainerCatalog()
        NeoPortalCatalog.ZopeFindAndApply( self
                            , catalog
                            , obj_metatypes=self.getNeoPortalElementContainerCataloggableMetaTypes()
                            , search_sub=1
                            , apply_func=catalog.catalog_object
                            , apply_path='' )
        elapse = time.time() - elapse
        c_elapse = time.clock() - c_elapse

    security.declareProtected( NeoPortalManagePortal, 'initializeNeoPortalElementContainerCatalog' )
    def initializeNeoPortalElementContainerCatalog( self ):
        """
            Init catalog
        """         
        self._initNeoPortalElementContainerCatalog()

    security.declarePrivate( 'getNeoPortalElementContainerCatalog' )
    def getNeoPortalElementContainerCatalog( self ):
        """
            Return Internal Catalog
        """         

        return self    

    security.declarePrivate( '_setObject' )
    def _setObject(self, id, object, roles=None, user=None, set_owner=1, no_id_check=0):
        """
            Custom ID's not allowed for a NeoPortal Element Container
        """
        
        if no_id_check:
            BaseFolder._setObject(self, id, object, roles=None, user=None, set_owner=1)
            return
            
        e_type = object.meta_type
        
        if not e_type or not getattr( object, 'neoportal_element', 0):
            self.np_log(message="<em>%s</em> cannot be added to this %s."  % ( id, self.meta_type ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="<em>%s</em> cannot be added to this %s.", tup=( id, self.meta_type ) )
            raise 'NeoPortalError', message

        neoportal_element_prefix = getattr( object, 'neoportal_element_prefix', '')
        
        if re.compile( r'^[a-z]+\_$', re.IGNORECASE ).search( neoportal_element_prefix ) is None:
            self.np_log(message="<em>%s</em> has no or invalid <em>neoportal_element_prefix</em> value."  % ( id, ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="<em>%s</em> has no or invalid <em>neoportal_element_prefix</em> value.", tup=( id, ) )
            raise 'NeoPortalError', message
            
        num = self.resetElementNumbers( e_type = e_type ) 

        id = neoportal_element_prefix + str( num )
        object.id = id
        object.__name__ = id

        BaseFolder._setObject(self, id, object, roles=None, user=None, set_owner=1)

    security.declarePrivate( '_delObject' )
    def _delObject( self, id, dp=1 ):
        """
            Uncatalog object before deletion
        """

        BaseFolder._delObject( self, id, dp=1 )        

    security.declarePrivate( 'addToNeoPortalElementContainerCatalog' )
    def addToNeoPortalElementContainerCatalog( self, object ):
        """
            Index object
        """
        
        catalog = self.getNeoPortalElementContainerCatalog()
        url = catalog._getObjectURL( object )
        catalog.catalog_object( object, url )    

    security.declarePrivate( '_getObjectURL' )
    def _getObjectURL( self, ob ):
        """
            Return Object URL
        """
        
        path = list( ob.getPhysicalPath() )
        path = path[ path.index( self.getNeoPortalElementContainerCatalog().getId() )+1: ]
        return '/'+ '/'.join( path )
        
    security.declarePrivate( 'deleteFromNeoPortalElementContainerCatalog' )
    def deleteFromNeoPortalElementContainerCatalog( self, object ):
        """
            Unindex object
        """
        
        catalog = self.getNeoPortalElementContainerCatalog()
        url = catalog._getObjectURL( object )
        catalog.uncatalog_object( url )
        
    security.declarePrivate( 'updateNeoPortalElementContainerCatalogIndex' )
    def updateNeoPortalElementContainerCatalogIndex( self, object ):
        """
            Reindex object
        """
        
        self.getNeoPortalElementContainerCatalog().deleteFromNeoPortalElementContainerCatalog( object )        
        self.getNeoPortalElementContainerCatalog().addToNeoPortalElementContainerCatalog( object )        
        
    security.declareProtected( NeoPortalModifyPortalContent, 'reindexObject' )
    def reindexObject( self, idxs=[] ):
        """
            Reindex the object in the portal catalog.
        """
        try:
            if idxs == []:
                # Update the modification date.
                if hasattr(aq_base(self), 'notifyModified'):
                    self.notifyModified()
            catalog = getToolByName(self, 'portal_catalog', None)
            if catalog is not None:
                catalog.reindexObject(self, idxs=idxs)                 
        except:
            pass
            
    security.declarePrivate( '_reorderNeoPortalElements' )
    def _reorderNeoPortalElements( self, element_prefix, insert_object=None, org_index=None, new_index=None ):
        """
            Reorder element numbers starting from 'element_number'
        """

        if not self._np_element_renumbering_allowed: return

        id_nums = [ int( element.split( '_' )[1] ) for element in self.objectIds() if element.startswith('%s_' % element_prefix ) ]
        if not id_nums: return
        
        id_nums.sort()
        
        reset_idx = None
        if insert_object is not None:
            
            if org_index < new_index and new_index > max( id_nums ): 
                new_index = max( id_nums )
        
            reset_idx = min( org_index , new_index )

        else:
            for num in xrange( 1, max( id_nums ) ):
                if num not in id_nums:
                    reset_idx = num - 1
                    break

        if reset_idx is None:
            return
        
        elements = [ element for element in self.objectValues() if element.getId().startswith('%s_' % element_prefix ) ]

        elements.sort(lambda x,y: cmp(int(x.getId().split('_' )[1]), int(y.getId().split('_' )[1])))

        for element in elements:
            self._delObject( element.getId() )

        if insert_object is not None:
            elements.insert( new_index, insert_object )

        element_number = 0
                
        for element in elements:
            element_number = element_number + 1
            id = element_prefix  + '_' + str( element_number )            
            element = aq_base( element )
            element.id = id
            element.__name__ = id

            self._setOb( id, element )
            
        self._initNeoPortalElementContainerCatalog()            
                    
    security.declarePrivate( 'manage_renameObject' )
    def manage_renameObject(self, id, new_id, REQUEST=None):
        """
            Call '_sebOb' instead of '_setObject' 
        """

        if re.compile( r'^[a-z]+\_[0-9]+$', re.IGNORECASE ).search( id ) is None:
            self.np_log(message="Id <em>%s</em> is invalid. "  % ( id, ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Id <em>%s</em> is invalid.", tup=( id, ) )
            raise 'NeoPortalError', message

        if re.compile( r'^[a-z]+\_[0-9]+$', re.IGNORECASE ).search( new_id ) is None:
            self.np_log(message="Id <em>%s</em> is invalid. "  % ( new_id, ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Id <em>%s</em> is invalid.", tup=( new_id, ) )
            raise 'NeoPortalError', message
        
        if self._np_element_renumbering_allowed:

            obj = self._getOb( id )
            self._delObject( id )
            self._reorderNeoPortalElements( element_prefix=id.split('_')[0], insert_object=obj, org_index=int( id.split('_')[1] ) - 1, new_index=int( new_id.split('_')[1] ) - 1 )
            
        else:
            try: self._checkId( new_id )
            except: raise 'CopyError', MessageDialog(
                          title='Rename Error',
                          message=sys.exc_info()[1],
                          action ='manage_main')
            
            ob=self._getOb(id)
            if not ob.cb_isMoveable():
                raise CopyError, eNotSupported % id
            self._verifyObjectPaste(ob)
            try:    ob._notifyOfCopyTo(self, op=1)
            except: raise 'CopyError', MessageDialog(
                          title='Rename Error',
                          message=sys.exc_info()[1],
                          action ='manage_main')
            self._delObject(id)
            ob = aq_base(ob)
            ob._setId(new_id)
    
            # Note - because a rename always keeps the same context, we
            # can just leave the ownership info unchanged.
            self._setOb(new_id, ob )

        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1)
        return None        

    security.declareProtected( NeoPortalView, 'getNeoPortalElements' )
    def getNeoPortalElements( self, e_type=None ):
        """
            Wrapper to catch exception (caused by catalog corruption)
        """ 
        try:
            return self._getNeoPortalElements( e_type=e_type )
        except:
            self.initializeNeoPortalElementContainerCatalog()
            return self._getNeoPortalElements( e_type=e_type )
                    
    security.declarePrivate( '_getNeoPortalElements' )
    def _getNeoPortalElements( self, e_type=None ):
        """
            Return the named Elements
        """ 

        if type( e_type ) == type ( '' ):
            return self.searchResults( meta_type=e_type, sort_on='getNeoPortalElementNumber' )
        elif type( e_type ) == type ( [] ):
            results = []            
            for t in e_type:
                results.extend( self.searchResults( meta_type=e_type
                                , sort_on='getNeoPortalElementNumber' ) )
    
            new_results = []
    
            seen={} # eliminate duplicates
            for res in results:
                if not hasattr(res, 'getObject'): continue            
                if seen.has_key( res.getURL() ): continue
                seen[ res.getURL() ]=1
                new_results.append( res )
    
            return new_results            
            
        else:
            return self.searchResults( sort_on='getNeoPortalElementNumber' )        

    security.declareProtected( NeoPortalView, 'getNeoPortalElement' )
    def getNeoPortalElement( self, id ):
        """
            Return the named Element
        """ 
        
        try:
            return getattr( self, id )
        except:
            return None
    
    security.declarePrivate( '_get_id' )
    def _get_id( self, id ):
        """
            Return new ID's for pasted elements
        """     
        try:
            element_prefix, element_num = id.split( '_' )
        except Exception, e:
            self.np_log(message="Bad non-sequenceable id '<em>%s</em>' in '<b>%s</b> (%s)'" % ( id, self.getId(), e ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Bad non-sequenceable id '<em>%s</em>' in '<b>%s</b> (%s)'", tup=( element.getId(), self.getId(), e ) )            
            raise 'NeoPortalError', message
        
        next_idx = self.resetElementNumbersByPrefix( e_prefix = element_prefix )

        return element_prefix + '_' + str( next_idx )

    security.declareProtected( NeoPortalModifyPortalContent, 'resetElementNumbersByPrefix' )        
    def resetElementNumbersByPrefix( self, e_prefix ):
        """
            Recaculate Element Numbers by prefix
        """
        elements = self.objectIds()
        try:
            id_nums= [ int( element.split( '_' )[1] ) for element in elements if element.startswith('%s_' % e_prefix ) ]

        except Exception, e:
            self.np_log(message="Bad non-sequenceable id '<em>%s</em>' in '<b>%s</b> (%s)'" % ( id, self.getId(), e ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Bad non-sequenceable id '<em>%s</em>' in '<b>%s</b> (%s)'", tup=( element.getId(), self.getId(), e ) )            
            raise 'NeoPortalError', message
        if not self._np_element_renumbering_allowed:

            if id_nums:
                return max( id_nums ) + 1
            else:
                return 1

        if not len( elements ): return 1
        else: return self.resetElementNumbers( e_type = self._getOb( elements[0] ).meta_type, e_prefix=e_prefix )        
               
    security.declareProtected( NeoPortalModifyPortalContent, 'resetElementNumbers' )        
    def resetElementNumbers( self, e_type=None, e_prefix=None ):
        """
            Recaculate Element Numbers in case an element gets deleted
        """
        
        if e_type is None and e_prefix is None:
            self.np_log(message="Element type or element prefix should be specified.", summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Element type or element prefix should be specified.")
            raise 'NeoPortalError', message
        
        element_prefix = e_prefix
        
        elements= self.objectIds()
        try:
            if element_prefix is not None:
                id_nums= [ int( element.split( '_' )[1] ) for element in self.objectIds() if element.startswith('%s_' % e_prefix ) ]
            else:
                id_nums=[]
                for element in elements:
                    if self._getOb( element ).meta_type == e_type:
                        element_prefix, element_number = element.split( '_' )
                        id_nums.append( int( element_number ) )
                    
        except Exception, e:
            self.np_log(message="Bad non-sequenceable id in '<b>%s</b> (%s)'" % ( self.getId(), e ), summary='NP_ElementContainer', severity='ERROR')
            message = self.getNeoPortalMessage( message="Bad non-sequenceable id in '<b>%s</b>(%s).'", tup=( self.getId(), e ) )
            raise 'NeoPortalError', message
            
        if not self._np_element_renumbering_allowed:        
            #
            # sometimes element renubmering is a real bad idea: set _np_element_renumbering to false to turn it off
            #
            if id_nums:
                return max( id_nums ) + 1
            else:
                return 1

        num_elements = len( id_nums )
        if not num_elements: return 1
                
        self._reorderNeoPortalElements( element_prefix=element_prefix )

        return num_elements + 1

    security.declareProtected( NeoPortalView, 'getNeoPortalElementContainer' )
    def getNeoPortalElementContainer( self ):
        """
            Return the container object
        """
        
        if hasattr( aq_base( self ) ,  '_np_top_most_element_container') and self._np_top_most_element_container:
            return self
        else:
            return self.aq_inner.aq_parent.getNeoPortalElementContainer()
           
InitializeClass( NeoPortalElementContainer )

class NeoPortalAdvancedElementContainer( NeoPortalElementContainer ):
    """ 
        NeoPortal Advanced Element Container
    """

    neoportal_advanced_element_container = 1
    
    security = ClassSecurityInfo()

InitializeClass( NeoPortalAdvancedElementContainer )