#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_EmbeddableContent.py,v 1.10 2003/03/26 18:18:09 wankyu Exp $

Description

    NeoPortal Embeddable Content can embed or be embedded in other objects.
    
    NeoPortal Embeddable content will only be usable by CMF prodcuts or NeoBoard
    
"""

from Globals import InitializeClass, PersistentMapping
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from Acquisition import aq_base
from AccessControl.ZopeGuards import guarded_getattr
from zExceptions import Unauthorized

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

try:
    from Products.CMFCore.utils import _getViewFor
    CMF_INSTALLED=1
except ImportError:
    CMF_INSTALLED=0

import re
    
class NeoPortalEmbeddableContent( NeoPortalBaseItem ):
    """
        NeoPortal Embeddable Content mix-in
    """

    neoportal_embeddable_content = 1

    # the object can be embedded in other objects
    _np_content_can_be_embedded = 1
    
    # the object can embedded objects
    _np_content_can_embed = 0
    
    # prefix/suffix used for embedding objects - prefix + object_id + suffix: __(MyImage)__
    
    _np_embedding_prefix = r'__('
    _np_embedding_suffix = r')__'

    security = ClassSecurityInfo()
        
    
    security.declarePrivate( '_getEmbeddingPrefix' )
    def _getEmbeddingPrefix( self ):
        """
            Return embedding marker prefix
        """

        return self._np_embedding_prefix

    security.declarePrivate( '_getEmbeddingSuffix' )
    def _getEmbeddingSuffix( self ):
        """
            Return embedding marker suffix
        """
                
        return self._np_embedding_suffix

    security.declarePrivate( '_getObjectRelativePath' )
    def _getObjectRelativePath( self, obj ):
        """
            Return object by relative path
        """
                
        utool = getattr( self, 'portal_url', None )
        if utool is None:
            return None
            
        return utool.getRelativeContentURL( obj )

    security.declarePrivate( '_getAttrByPath' )
    def _getAttrByPath( self, path ):
        """
            Return attribute by path
        """
        if not path: return None
        parts = list( path.split( '/' ) )
        utool = getattr( self, 'portal_url', None )
        
        if utool is None:
            return None        
        
        portal = utool.getPortalObject()
        base = portal
        
        for part in parts:
            obj = guarded_getattr( base, part, None )
            
            if obj is None:
                break
            else:
                base = obj
                
        return obj
    
    security.declarePrivate( '_checkEmbedContainer' )
    def _checkEmbedContainer( self ):
        """
            Check to see if the object has embed properties
        """
       
        if not hasattr( self, '_np_embedding_objects' ):
            self._np_embedding_objects = PersistentMapping()
            
        if not hasattr( self, '_np_embedded_objects' ):
            self._np_embedded_objects = PersistentMapping()

    security.declareProtected( NeoPortalView, 'getEmbeddingObjects' )      
    def getEmbeddingObjects( self ):
        """
            Return Embedding Objects
        """                    
        self._checkEmbedContainer()
        
        self.np_log( message="%s's _np_embedding_objects: %s" % ( self.getId(), str( self._np_embedding_objects ) ), summary='NP_EmbeddableContent', severity='DEBUG' )
        
        objs = []
        for path in self._np_embedding_objects.keys():
            obj = self._getAttrByPath(path)
            if obj:
                objs.append(obj)
            else:
                self.np_log( message="%s's dead embedding obj: %s" % ( self.getId(), path ), summary='NP_EmbeddableContent', severity='DEBUG' )            
                del self._np_embedding_objects[path]            
                self.np_log( message="%s's _np_embedding_objects: %s" % ( self.getId(), str( self._np_embedding_objects ) ), summary='NP_EmbeddableContent', severity='DEBUG' )
            
        return objs

    security.declareProtected( NeoPortalView, 'getEmbeddedObjects' )       
    def getEmbeddedObjects( self ):
        """
            Return Embedded Objects
        """        
        self._checkEmbedContainer()
        
        self.np_log( message="%s's _np_embedded_objects: %s" % ( self.getId(), str( self._np_embedded_objects ) ), summary='NP_EmbeddableContent', severity='DEBUG' )
        
        objs = []
        for path in self._np_embedded_objects.keys( ):
            obj = self._getAttrByPath(path)
            if obj:
                objs.append(obj)
            else:
                self.np_log( message="%s's dead embedded obj: %s" % ( self.getId(), path ), summary='NP_EmbeddableContent', severity='DEBUG' )            
                del self._np_embedded_objects[path]
                self.np_log( message="%s's _np_embedded_objects: %s" % ( self.getId(), str( self._np_embedded_objects ) ), summary='NP_EmbeddableContent', severity='DEBUG' )
                
        return objs

    security.declareProtected( NeoPortalView, 'getEmbeddingObjectIds' )       
    def getEmbeddingObjectIds( self ):
        """
            Return Embedding Object Ids
        """
        self._checkEmbedContainer() 

        return [ obj.getId() for obj in self.getEmbeddingObjects() ]

    security.declareProtected( NeoPortalView, 'getEmbeddedObjectIds' )       
    def getEmbeddedObjectIds( self ):
        """
            Return Embedded Objects Ids
        """ 
        self._checkEmbedContainer()
        
        return [ obj.getId() for obj in self.getEmbeddedObjects() ]

    security.declareProtected( NeoPortalView, 'getLastEmbeddedDate' )     
    def getLastEmbeddedDate( self, abs_url ):
        """
            Return embedding date
        """
        
        if self._np_embedded_objects.has_key( abs_url ):
            return self._np_embedded_objects[ abs_url ]
        else:
            return None

    security.declareProtected( NeoPortalView, 'contentCanBeEmbedded' ) 
    def contentCanBeEmbedded( self ):
        """
            Tell if content can be embedded in other objects
        """        

        if not getattr( self, 'neoportal_embeddable_content', 0 ): return 0
        if getattr( self, 'neoportal_cacheable_content', 0 ) and not self.isContentCacheable(): return 0

        return self._np_content_can_be_embedded

    security.declareProtected( NeoPortalView, 'contentCanEmbed' )    
    def contentCanEmbed( self ):
        """
            Tell if content can embed other objects
        """
        
        if not getattr( self, 'neoportal_embeddable_content', 0 ): return 0
               
        return self._np_content_can_embed    

    security.declarePrivate( '_embedObject' )
    def _embedObject( self, embedded_obj ):
        """
            Embed object
        """

        self.np_log( message="%s's _embedObject called" % self.getId(), summary='NP_EmbeddableContent', severity='DEBUG' )
        
        if not self.contentCanEmbed():
            self.np_log( message="%s can't embed objects" % self.getId(), summary='NP_EmbeddableContent', severity='DEBUG' )
            return
        
        self._checkEmbedContainer()

        self._np_embedded_objects[self._getObjectRelativePath(embedded_obj)] = DateTime()
        embedded_obj._notifyEmbedded( self)

    security.declarePrivate( '_delEmbeddedObject' )
    def _delEmbeddedObject( self, embedded_obj ):
        """
            Delete embedded object
        """
        
        self.np_log( message="%s's _delEmbeddedObject called" % self.getId(), summary='NP_EmbeddableContent', severity='DEBUG' )
        
        if not self.contentCanEmbed():
            self.np_log( message="%s can't embed objects" % self.getId(), summary='NP_EmbeddableContent', severity='DEBUG' )
            return 0
        
        self._checkEmbedContainer()

        try:                        
            del self._np_embedded_objects[ self._getObjectRelativePath( embedded_obj ) ]
            embedded_obj._notifyDisembedded( self )
            self.np_log( message="%s's embedded obj deleted: %s" % ( self.getId(), embedded_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        except KeyError:
            self.np_log( message="%s's embedded obj not found: %s" % ( self.getId(), embedded_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )                    
            pass

    security.declarePrivate( '_notifyEmbedded' )
    def _notifyEmbedded( self, embedding_obj ):
        """
            Notify self that some object has embedded it
        """
        self.np_log( message="%s's _notifyEmbedded called by %s" % ( self.getId(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        if not self.contentCanBeEmbedded():
            return
        
        self._checkEmbedContainer() 
        
        if not self.contentCanBeEmbedded():
            return 0

        self._np_embedding_objects[ self._getObjectRelativePath( embedding_obj ) ] = DateTime()
        self.np_log( message="%s embedded by obj on %s: %s" % ( self.getId(), DateTime(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )                    

    security.declarePrivate( '_notifyDisembedded' )
    def _notifyDisembedded( self, embedding_obj ):
        """
            Notify self that some object has disembedded it
        """
                
        self.np_log( message="%s's _notifyDisembedded called by %s" % ( self.getId(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG')
        self._checkEmbedContainer() 
                    
        self._selfDisembedded( embedding_obj )

    security.declarePrivate( '_selfDisembedded' )
    def _selfDisembedded( self, embedding_obj ):
        """
            Respond to a notification that embedding_obj disembedded self
        """
        
        self.np_log( message="%s's _selfDisembedded called by %s" % ( self.getId(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        if not self.contentCanBeEmbedded():
            return

        self._checkEmbedContainer()            
            
        try:            
            del self._np_embedding_objects[ self._getObjectRelativePath( embedding_obj ) ]
            self.np_log( message="%s's been disembedded by %s" % ( self.getId(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        except KeyError:
            self.np_log( message="%s's embedding obj not found: %s" % ( self.getId(), embedding_obj.getId() ), summary='NP_EmbeddableContent', severity='ERROR' )

    security.declarePrivate( '_embeddedObjectChanged' )
    def _embeddedObjectChanged( self, embedded_obj ):
        """
            Respond to changes in an embedded object
        """
        self.np_log( message="%s's _embeddedObjectChanged called by %s" % ( self.getId(), embedded_obj.getId() ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        self._checkEmbedContainer()        

        if not self.contentCanEmbed( ):
            return

        self._updateEmbeddedContent( embedded_obj )

    security.declarePrivate( '_notifyChangesInEmbededObject' )
    def _notifyChangesInEmbededObject( self ):
        """
            Notifies changes in self to embedding objects
        """
        
        self.np_log( message="%s's _notifyChangesInEmbededObject called" % ( self.getId(), ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        
        self._checkEmbedContainer()        

        if not self.contentCanBeEmbedded():
            return
        
        for e_obj in self.getEmbeddingObjects():
            e_obj._embeddedObjectChanged( self )

    security.declarePrivate( '_updateEmbeddedContent' )
    def _updateEmbeddedContent( self, id ):
        """
            Called when an embeded object notifies changes in itself by calling _notifyChangesInEmbededObject()
        """

        self._updateNeoPortalContent()

    security.declarePrivate( '_getEmbeddedElementRegEx' )
    def _getEmbeddedElementRegEx( self ):
        """
            Return Regular Expression used to extract embeds
        """
        return re.escape( self._getEmbeddingPrefix()) + r'[ \-\_\.a-z0-9\/]+' + re.escape( self._getEmbeddingSuffix())

    security.declarePrivate( '_stripAllEmbedMarkers' )
    def _stripAllEmbedMarkers( self, content ):
        """
            Remove all embed markers from content
        """
        return re.sub( self._getEmbeddedElementRegEx(), '', content )

    security.declarePrivate( '_checkPermissionFor' )
    def _checkPermissionFor( self, content ):
        """
            Check the current user's permissions for embedding content
        """

        if hasattr( content, 'isNeoPortalManager') and content.isNeoPortalManager(): return 1
        if hasattr( content, 'isNeoPortalContentOwner') and content.isNeoPortalContentOwner(): return 1
        
        try:
            review_state = self.portal_workflow.getInfoFor( content, 'review_state', '')
            if review_state.lower() != 'published' and review_state.lower() != 'public':
                return 0
            return 1
        except:
            return 1

    security.declarePrivate( '_pruneDeadReferences' )
    def _pruneDeadReferences( self ):
        """
            Find removed references and kill them accordingly
        """

        # first define embedding pattern
        _embedded_element_re = self._getEmbeddedElementRegEx()
        e_pattern = re.compile( _embedded_element_re, re.IGNORECASE )
        
        # strip suffix/prefix pairs from found embed markers
        pre = len( self._getEmbeddingPrefix() )
        suf = len( self._getEmbeddingSuffix() )
        
        content = self.description
        
        if hasattr( aq_base( self ), 'text'):
            content = self.text + content
        
        e_ids = [ elem.strip()[pre:-suf] for elem in e_pattern.findall( content ) ]
                
        dead_refs = [ e_obj for e_obj in self.getEmbeddedObjects() if self._getObjectRelativePath( e_obj ) not in e_ids ]
        for dead_ref in dead_refs:
            self._delEmbeddedObject( dead_ref )

    security.declarePrivate( '_renderEmbeddedObjects' )
    def _renderEmbeddedObjects( self, content, REQUEST=None, **kw ):
        """
            Render embedded objects
        """       

        self.np_log( message="_renderEmbeddedObjects called by %s" % self.getId(), summary='NP_EmbeddableContent', severity='DEBUG' )
        
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        # first define embedding pattern
        _embedded_element_re = self._getEmbeddedElementRegEx()
        e_pattern = re.compile( _embedded_element_re, re.IGNORECASE )
        
        # strip suffix/prefix pairs from found embed markers
        pre = len( self._getEmbeddingPrefix() )
        suf = len( self._getEmbeddingSuffix() )
        e_ids = [ elem.strip()[pre:-suf] for elem in e_pattern.findall( content ) ]
        
        self._pruneDeadReferences()
                            
        self.np_log( message="%s's _renderEmbeddedObjects found markers: %s" % ( self.getId(), e_ids ), summary='NP_EmbeddableContent', severity='DEBUG' )            
        
        for e_id in e_ids:

            e_re = re.escape( self._getEmbeddingPrefix() ) + e_id + re.escape( self._getEmbeddingSuffix() )
        
            if e_id == self._getObjectRelativePath( self ):
                # why would anyone try to embed self?
                content = re.sub(e_re, self._getEmbeddingPrefix() + '<b>CANNOT_EMBED_SELF:</b><em>%s</em>' % ( e_id, ) + self._getEmbeddingSuffix(), content )
                continue
            
            e_obj = guarded_getattr( self, e_id, None )
            if e_obj is None:
                e_obj = self._getAttrByPath( e_id )
            
            if e_obj is not None:
                try:
                    if not hasattr( e_obj, 'contentCanBeEmbedded' ) or not e_obj.contentCanBeEmbedded():
                        self.np_log( message="Embedding not supported: %s" % ( e_id, ), summary='NP_EmbeddableContent' , severity='DEBUG' )            
                        content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>REQUESTED_OBJECT_DOES_NOT_SUPPORT_EMBEDDING:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )
                    else:
                        if not self._checkPermissionFor( e_obj ):
                            self.np_log( message="Not authorized: %s" % ( e_id, ), summary='NP_EmbeddableContent' , severity='DEBUG' )            
                            content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>CANNOT_EMBED_UNAUTHORIZED_OBJECT_IN_THIS_CONTEXT:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )
                            continue
                            
                        if self.getId() in e_obj.getEmbeddedObjectIds( ):
                            # cross embedding leads to something really bad: simply put, Ba~~~aaaad idea!
                            self.np_log( message="Cross embedding not allowed: %s" % ( e_id, ), summary='NP_EmbeddableContent' , severity='DEBUG' )            
                            content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>CANNOT_CROSS_EMBED:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )
                            continue

                        e_obj = e_obj.__of__( self )
                        view = guarded_getattr( e_obj, 'call_view', None )
                        
                        if view is None and CMF_INSTALLED:
                            view = _getViewFor( e_obj, 'call_view' )

                        view = apply( view, ( REQUEST, ) )

                        content = re.sub( e_re, view, content )
                        
                        self._embedObject( e_obj )
                        
                except Unauthorized, e:
                    self.np_log( message="Unauthorized %s (%s)" % ( e_id, e ), summary='NP_EmbeddableContent' , severity='DEBUG' )            
                    content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>NOT_AUTHRIZED_TO_EMBED_REQUESTED_OBJECT_IN_THIS_CONTEXT:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )
                except AttributeError, e:
                    self.np_log( message="No call view for %s (%s)" % ( e_id, e ), summary='NP_EmbeddableContent' , severity='DEBUG' )            
                    content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>REQUESTED_OBJECT_DOES_NOT_SUPPORT_EMBEDDING_NO_CALL_VIEW:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )
            else:
                self.np_log( message="%s not found in %s" % ( e_id, self.getId() ), summary='NP_EmbeddableContent' , severity='INFO' )            
                content = re.sub( e_re, self._getEmbeddingPrefix() + '<b>REQUESTED_OBJECT_NOT_FOUND:</b><em>%s</em>' % (e_id,) + self._getEmbeddingSuffix(), content )

        return content

InitializeClass( NeoPortalEmbeddableContent )