#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_IconizableContent.py,v 1.7 2003/03/26 18:18:09 wankyu Exp $
Description

    NeoPortal Iconizable Content can convert simple text notations into icons: emoticons, for example
 
"""

from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import os, re

TEXT_ICONS_FILE = package_home( globals() ) + '/data/text_icons.props'           
MIME_ICONS_FILE = package_home( globals() ) + '/data/mime_icons.props'
MEDIA_ICONS_FILE = package_home(globals()) + '/data/media_icons.props'           

class NeoPortalIconizableContent( NeoPortalBaseItem):
    """
        Mix-in that converts simple text notations into icons
    """
    
    neoportal_iconizable_content = 1

    _np_content_icons_map = {}
    _np_mime_icons_map = {}

    _np_media_icons_map = {}
    _np_media_extensions_map = {}
    
    _np_renderers = ['_cookContent', '_iconizeNeoPortalContent']
    
    security = ClassSecurityInfo()
    
    security.declarePublic( 'contentIconizationAllowed' )        
    def contentIconizationAllowed( self ):
        """
           Tell if content iconization is allowed
        """

        if not getattr( self, 'neoportal_iconizable_content', 0 ): return 0
        
        if hasattr( self, 'np_content_iconization_allowed' ):
            return self.np_content_iconization_allowed
        else:
            try:
                return self.neoportal_properties.isContentIconizable( self )
            except:
                return 1
            
    security.declarePrivate( 'getNeoPortalContentIconsMap' )
    def getNeoPortalContentIconsMap( self ):
        """
           Return the content icons map
        """
        
        if not len( self._np_content_icons_map ): 
            self.initNeoPortalContentIcons()
                    
        return NeoPortalIconizableContent._np_content_icons_map

    security.declarePrivate( 'getNeoPortalMimeIconsMap' )
    def getNeoPortalMimeIconsMap( self ):
        """
           Return the mime icons map
        """
        
        if not len( self._np_mime_icons_map ): 
            self.initNeoPortalMimeIcons()
                            
        return NeoPortalIconizableContent._np_mime_icons_map
        
    security.declarePublic( 'getNeoPortalContentIconNotations' )
    def getNeoPortalContentIconNotations( self ):
        """
           Return the content icon text notaiions
        """        

        icon_map = self.getNeoPortalContentIconsMap()
                
        return icon_map.keys()

    security.declarePublic( 'getNeoPortalContentIconNotations' )
    def getNeoPortalContentIconByNotation( self, notation ):
        """
           Return the content icon text notaiions
        """
        
        icon_map = self.getNeoPortalContentIconsMap()
            
        try:
            return icon_map[notation]
        except:
            return None

    security.declarePrivate( 'getIconURL' )
    def getNeoPortalIconURLPrefix( self ):
        """
            Override this method to provide a different url than the object's absolute_url
        """

        return self.absolute_url()

    security.declarePrivate( 'iconizeNeoPortalContent' )
    def iconizeNeoPortalContent( self, content='', **kw):
        """
           Return the given content after converting every known text notations into icons
        """
        
        if not self.contentIconizationAllowed():
            self.np_log( message="iconization not allowed: %s" % self.getId(), summary='NP_IconizableContent', severity='DEBUG' )
            return content
        
        icon_map = self.getNeoPortalContentIconsMap()

        for notation in icon_map.keys():
            notation_pattern = re.compile( r"(\s+|^)" + re.escape( notation.replace('\\', '') ) + r"(\s+|$)", re.IGNORECASE )
            try:
                content = notation_pattern.sub( r"\1" + "<img src=\"%s/%s\" class=\"np_text_icon\" alt=\"%s\">" % ( self.getNeoPortalIconURLPrefix(), icon_map[notation], icon_map[notation] ) + r"\2", content )
            except Exception, e: 
                self.np_log( message="iconization of %s failed: %s" % ( self.getId(), e ), summary='NP_IconizableContent', severity='DEBUG' )
                pass
            
        return content
            
    security.declareProtected( NeoPortalManagePortal, 'initNeoPortalContentIcons' )
    def initNeoPortalContentIcons( self ):
        """
            Initialize Content Icons Map
        """
        f = open( TEXT_ICONS_FILE, 'rb' )
        lines = [ line.strip().lower() for line in f.readlines() ]
        
        icon_map = self._np_content_icons_map
        
        for line in lines:
            if line.startswith( '#' ): continue # comment
            
            try:
                notation = line.split('___')[0].strip()
                icon = line.split('___')[1].strip()
            except:
                continue
            
            if notation.find(',') > 0:
                for n in notation.split( ',' ):
                    n = n.strip()
                    if n:
                        icon_map[n] = icon
            else:
                icon_map[notation] = icon
        
    security.declareProtected( NeoPortalManagePortal, 'initNeoPortalMimeIcons')
    def initNeoPortalMimeIcons( self ):
        """
            Initialize Mime Icons
        """
        
        f = open( MIME_ICONS_FILE, 'rb' )
        lines = [ line.strip().lower() for line in f.readlines() ]

        #unknown/*:mime_unknown */
        #image/*:mime_img       */
        #text/*:mime_text       */
        #text/html:mime_html    */
        
        icon_map = self._np_mime_icons_map
        for line in lines:
            if line.startswith('#'): continue # comment
            
            mtype, icon = line.split(':')
            
            if not icon or not mtype: continue # bad format

            icon_map[mtype] = icon

    security.declareProtected( NeoPortalView, 'getMimeTypeIcon')
    def getMimeTypeIcon( self, mime_type=None ):
            
        icon_map = self.getNeoPortalMimeIconsMap()
        
        if mime_type is None:
            return getattr( self, icon_map.get( 'unknown/*' ), '-' )

        icon = icon_map.get(mime_type, None)
        if icon:
            return getattr( self, icon )
        
        try:
            mtype, subtype  = mime_type.split('/', 2)
        except:
            return getattr( self, icon_map.get( 'unknown/*' ), '-' )
            
        icon = icon_map.get( mtype + '/' + '*', None )
        if icon:        
            return getattr( self, icon )
        else:
            return getattr( self, icon_map.get( 'unknown/*' ), '-' )            

    security.declarePublic( 'getMediaTypes' )    
    def getMediaTypes( self ):
        """
            Return Supported Media Types
        """
        types = [key for key in self.getNeoPortalMediaIconsMap().keys() if key != 'etc']
        types.sort()
        types.append('etc')
        
        return types

    security.declarePublic( 'getMediaExtensions' )
    def getMediaExtensions( self, media_type=None ):
        """
            Return Supported Media Extensions or requested type's extensions
        """
        
        if media_type:
            try:
                return self.getNeoPortalMediaExtensionsMap()[media_type]
            except:
                return None
        
        exts = []
        
        for ext in self.getNeoPortalMediaExtensionsMap().values():
            exts.extend( ext )
        
        exts.sort()
        
        return exts
        
    security.declarePublic( 'getNeoPortalMediaIconsMap' )        
    def getNeoPortalMediaIconsMap( self ):
        """
           Return the media icons map
        """

        if not len( self._np_media_icons_map ):
            self.initNeoPortalMediaIcons()
                    
        return self._np_media_icons_map

    security.declarePublic( 'getNeoPortalMediaExtensionsMap' )        
    def getNeoPortalMediaExtensionsMap( self ):
        """
           Return the media icons map
        """

        if not len( self._np_media_extensions_map ):
            self.initNeoPortalMediaIcons()
                    
        return self._np_media_extensions_map

    security.declareProtected( NeoPortalManagePortal, 'initNeoPortalMediaIcons' )
    def initNeoPortalMediaIcons( self ):
        """
            Initialize Media Icons Map
        """        
        f = open(MEDIA_ICONS_FILE, 'rb')
        lines = [line.strip().lower() for line in f.readlines()]
        
        icon_map = self._np_media_icons_map
        ext_map = self._np_media_extensions_map
        
        for line in lines:
            if line.startswith('#' ): continue # comment
            
            try:
                tokens = line.split(':')
                name = tokens[0].strip()
                ext  = tokens[1].strip()
                icon = tokens[2].strip()
            except:
                continue
            
            icon_map[name] = icon
            
            if not ext: continue
            
            ext_map[name] = ext.split(',')

    security.declarePublic( 'getNeoPortalMediaIcon' )        
    def getNeoPortalMediaIcon( self, name ):
        """
           Return the requested media icon
        """
        
        icon_map = self.getNeoPortalMediaIconsMap()
                
        if icon_map.has_key(name ):
            return icon_map[name]

        return icon_map['etc']
            
InitializeClass( NeoPortalIconizableContent )

