#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_InlineComment.py,v 1.2 2003/03/26 18:18:09 wankyu Exp $
Description

    NeoPortal Inline Comment
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_CacheableContent import NeoPortalCacheableContent
from Products.NeoPortalLibrary.NP_IconizableContent import NeoPortalIconizableContent

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import os, mimetypes
    
class NeoPortalInlineComment( NeoPortalBaseItem, NeoPortalCacheableContent, NeoPortalIconizableContent ):
    """
        NeoPortal Inline Comment
    """
    _np_renderers = ['_cookContent', 'iconizeNeoPortalContent']
    
    meta_type='NeoPortal Inline Comment'
    icon='misc_/NeoPortalLibrary/np_inline_comment_icon'
    
    security = ClassSecurityInfo()
    
    def __init__( self, id, creator, text='', password='', email=''):
        self.id = id
        self.text=text
        self.creation_date=DateTime()
        self.creator = creator
        self.creator_email = email
        if not password:
            password = self._generatePassword()
        
        self.password = password
    
    security.declarePrivate( 'getInlineCommentView' )        
    def getInlineCommentView( self ):
        """
            Get inline comment view
        """        
        return self.np_inline_comment_view

    security.declarePrivate( 'getLostInlineCommentView' )        
    def getLostInlineCommentView( self ):
        """
            Get the view for comments that lost context (deleted, moved, etc)
        """        
        return self.np_lost_inline_comment_view    
        
    security.declarePrivate( 'getIconURL' )
    def getNeoPortalIconURLPrefix( self ):
        """
            Inline Comment should return its parent's url
        """

        return self.aq_inner.aq_parent.absolute_url()        

    security.declarePrivate( '_edit' )
    def _edit( self, text='', email='' ):
        """                
            Edit Comment
        """ 
         
        self.text=text
        self.creator_email = email
                        
        return None
        
InitializeClass( NeoPortalInlineComment )        