#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_InlineCommentableContent.py,v 1.11 2003/03/26 19:18:11 wankyu Exp $
Description

	NeoPortal Inline Commentable Content can hold inline comment objects
 
"""

from Globals import	InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from AccessControl.ZopeGuards import guarded_getattr
from BTrees.OOBTree import OOBTree

from Products.NeoPortalLibrary.NP_CacheableContent import NeoPortalCacheableContent
from Products.NeoPortalLibrary.NP_EmbeddableContent import NeoPortalEmbeddableContent
from Products.NeoPortalLibrary.NP_InlineComment import NeoPortalInlineComment
from Products.NeoPortalLibrary.NP_SpamBlockableObject import NeoPortalSpamBlockableObject
from Products.NeoPortalLibrary.NP_Utils import isEmailValid

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from Products.NeoPortalLibrary.NP_Utils import removeTags, getListOfParagraphs
import re

class NeoPortalInlineCommentableContent( NeoPortalSpamBlockableObject
                                        , NeoPortalCacheableContent
                                        , NeoPortalEmbeddableContent ):
    """
		Mix-in class which provides	methods enabling inline comments
    """

    neoportal_inline_commentable_content = 1
    
    _np_content_can_embed = 1

    _np_inline_comment_object_prefix = 'npinlinecomment'  # __(npinlinecomment_1)__
    _np_inline_comment_form_marker = 'npinlinecommentbtn' # __(npinlinecommentbtn_1)__
    
    _np_inline_comment_meta_type = 'NeoPortal Inline Comment'
    _np_max_inline_comment_size = 1000 # overridable
    
    _np_renderers = ['_cookContent', '_insertInlineCommentFormMarker', '_renderInlineComments', '_renderEmbeddedObjects']
    
    security = ClassSecurityInfo()

    security.declarePublic( 'inlineCommentsAllowed' )
    def inlineCommentsAllowed( self ):
        """
           Tell if inline comments are enabled
        """
        
        if not getattr( self, 'neoportal_inline_commentable_content', 0 ): return 0
        if getattr( self, 'neoportal_cacheable_content', 0 ) and not self.isContentCacheable(): return 0
        
        if hasattr( self, 'np_inline_comments_allowed' ):
            return self.np_inline_comments_allowed
        else:
            try:
                return self.neoportal_properties.isContentInlineCommentable( self )
            except:
                return 0
            
    security.declarePublic( 'getMaxInlineCommentSize' )        
    def getMaxInlineCommentSize( self ):
        """
           Return maximum number of characters allowed in a comment
        """
        if hasattr( self, 'np_max_inline_comment_size' ):
            return self.np_max_inline_comment_size
        else:
            try:
                return self.neoportal_properties.max_inline_comment_size
            except:
                return self._np_max_inline_comment_size
            
    security.declarePublic( 'getInlineCommentContainerURL' )    
    def getInlineCommentContainerURL( self ):
        """
           Return this object's absolute url
        """
        return self.absolute_url()
                    
    security.declarePrivate( 'getInlineCommentContainer' )        
    def getInlineCommentContainer( self ):
        """
            Return container that hold inline comments
        """
        if not hasattr( self, '_np_inline_comments' ):
            self._np_inline_comments = OOBTree()
        
        return self._np_inline_comments        
        
    security.declarePublic( 'getNeoPortalContentParagraph' )
    def getNeoPortalContentParagraph( self, index ):
        """
            Return a paragraph corresponding to a given index
        """
        index = int( index )
        if index:
            paragraphs = getListOfParagraphs( self.getRawNeoPortalContent() )
            try:
                # first remove any comment markers
                
                p = re.sub( self._getEmbeddedElementRegEx(), '', paragraphs[index-1] )
                
                # skip rendering inline comment markers
                renderers = [ r for r in self._getNeoPortalContentRenderers() if r != '_insertInlineCommentFormMarker' and r != '_renderInlineComments' ]
                return self._renderNeoPortalContent( content = p,  renderers = renderers)
            except:
                return ''
        else:
            return ''

    security.declarePublic( 'getNextCommentNum' )
    def getNextCommentNum( self ):
        """
            Return next inline comment number
        """     
        nums = [ int( comment.getId().split('_')[1] ) for comment in self.getInlineComments() ]
        if nums:
            return max(nums) + 1
        else:
            return 1            

    security.declarePrivate( '_getInlineCommentPrefix' )
    def _getInlineCommentPrefix( self ):
        """
            Return Inline Comment Prefix
        """
        return self._np_inline_comment_object_prefix
        
    security.declarePrivate( '_getInlineCommentMarkerRegEx' )
    def _getInlineCommentMarkerRegEx( self ):
        """
            Return Regular Expression used to extract inline comments
        """
        return re.escape( self._getEmbeddingPrefix() + self._getInlineCommentPrefix()) + r'[\_a-z0-9]+' + re.escape( self._getEmbeddingSuffix() )            

    security.declarePrivate( '_stripInlineCommentMarkers' )
    def _stripInlineCommentMarkers( self, content ):
        """
            Strip all inline comment markers
        """
        return re.sub( self._getInlineCommentMarkerRegEx(), '', content )            
            
    security.declareProtected( NeoPortalView, 'addInlineComment' )
    def addInlineComment( self, p_index, text='', password='', email='', REQUEST=None ):
        """
            Add Inline Comment
        """        
                
        id = self._getInlineCommentPrefix() + '_' + str( self.getNextCommentNum() )
        creator = self.getAuthenticatedNeoPortalUser( REQUEST=REQUEST )
        if creator is None or not hasattr( creator, 'getUserName' ):
            message = self.getNeoPortalMessage( message="Unknown user." )            
            if REQUEST is not None:
                return REQUEST['RESPONSE'].redirect(
                    self.absolute_url()
                    + "?portal_status_message=%s" % message
                    )
            else:
                raise 'NeoPortalError', message

        if email and not isEmailValid( email ):
            message = self.getNeoPortalMessage( message="Invalid email address." )            
            raise 'NeoPortalError', message
                
        comment = NeoPortalInlineComment( id=id, creator=creator.getUserName(), text=text, email=email )
        
        comment.setPassword( password )
        
        comment._setRemoteIP( REQUEST=REQUEST )
        self._insertInlineCommentMarker( id=id, p_index=p_index )
        self._addInlineComment( comment )
        self._updateNeoPortalContent()

        # spam check: DOS attack!
        self.validateDuplicateNeoPortalContentPosting( REQUEST=REQUEST )

    security.declarePublic( 'isInlineCommentEditable' )
    def isInlineCommentEditable( self, comment_id, REQUEST=None ):
        """
            Tell if the given comment is editable by the current user
        """
        
        comment = self.getInlineComment( id=comment_id )
        user = self.getAuthenticatedNeoPortalUser()
        
        if user is None or not hasattr( user, 'getUserName' ) or user.getUserName() != comment.Creator():
            if not hasattr( user, 'has_permission' ) or not user.has_permission( NeoPortalModifyPortalContent, comment):
                return 0
        return 1

    security.declareProtected( NeoPortalView, 'editInlineComment' )
    def editInlineComment( self, comment_id, text='', password='', email='', REQUEST=None ):
        """
            Edit Inline Comment
        """        
        
        comment = self.getInlineComment( id=comment_id )
        
        if comment.Creator() == 'Anonymous User' and not comment.checkPassword( password ):
            message = self.getNeoPortalMessage( message="You don't have permissions to edit this comment." )            
            raise 'NeoPortalError', message
        
        if not self.isInlineCommentEditable( comment_id ):
            message = self.getNeoPortalMessage( message="You don't have permissions to edit this comment." )            
            raise 'NeoPortalError', message

        if email and not isEmailValid( email ):
            message = self.getNeoPortalMessage( message="Invalid email address." )            
            raise 'NeoPortalError', message
        
        comment._edit( text=text, email=email )
        self._updateNeoPortalContent()

    security.declareProtected( NeoPortalView, 'deleteInlineComment' )
    def deleteInlineComment( self, comment_id, password='', check_permission=1, REQUEST=None ):
        """
            Delete Inline Comment
        """
       
        comment = self.getInlineComment( id=comment_id )
        
        if check_permission and comment.Creator() == 'Anonymous User' and not comment.checkPassword( password ):
            message = self.getNeoPortalMessage( message="You don't have permissions to delete this comment." )   
            raise 'NeoPortalError', message
        
        self._removeInlineComment( id=comment_id )
        self._updateNeoPortalContent()
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message="Deleted." )
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + '?portal_status_message=%s' % message
                )               

    security.declarePrivate( 'getInlineCommentFormMarker' )        
    def getInlineCommentFormMarker( self ):
        """
            Get inline comment form marker
        """        
        return self._np_inline_comment_form_marker

    security.declarePrivate( 'getInlineCommentButtonView' )        
    def getInlineCommentButtonView( self ):
        """
            Get inline comment button view
        """        
        return self.np_inline_comment_button
        
    security.declarePrivate( 'getInlineCommentMetaType' )        
    def getInlineCommentMetaType( self ):
        """
            Get inline comment meta type
        """        
        return self._np_inline_comment_meta_type

    security.declarePrivate( '_insertInlineCommentFormMarker' )
    def _insertInlineCommentFormMarker( self, content, skip=0, **kw ):
        """
            Insert inline comment form marker (an icon or a button) after every paragraph
        """

        if skip or not self.inlineCommentsAllowed(): return content

        paragraphs = getListOfParagraphs( content )

        new_paragraphs = []
        idx = 1

        form_marker = self._getEmbeddingPrefix() + self.getInlineCommentFormMarker() + '_%d' + self._getEmbeddingSuffix()

        embeds = []

        for p in paragraphs:
            if p.strip():
                embed = form_marker % (idx,) # comment_1, comment_2, etc...

                idx = idx + 1
                                
                new_paragraphs.append( p + embed )

        self.np_log( message="form markers: %s" % embeds, summary='NP_InlineCommentableContent', severity='DEBUG')

        return ''.join( new_paragraphs )

    security.declarePrivate( '_insertInlineCommentMarker' )
    def _insertInlineCommentMarker( self, id, p_index, **kw ):
        """
            Insert inline marker for a given comment
        """
        
        paragraphs = getListOfParagraphs( self.getRawNeoPortalContent() )
        
        # insert __(comment)__
        comment_marker = self._getEmbeddingPrefix() + '%s' + self._getEmbeddingSuffix()
        
        embed = comment_marker % (id,) # c_1, c_2, etc...
        
        try:
            p = paragraphs[ int( p_index ) - 1 ]
            p = p + embed
            paragraphs[ int( p_index ) - 1 ] = p

        except IndexError, ValueError:
            return 0
        
        self.setRawNeoPortalContent( ''.join( paragraphs ) )
        
        return 1

    security.declarePrivate( '_removeInlineComment' )
    def _removeInlineComment( self, id ):
        """
            Remove the given comment from container
        """
        content = self.getRawNeoPortalContent()
        
        # remove newline if need be
        r_re = re.escape( self._getEmbeddingPrefix() ) + id + re.escape( self._getEmbeddingSuffix() )
        content = re.sub(r_re, '', content)

        r_re = re.escape( self._getEmbeddingPrefix() ) + id + re.escape( self._getEmbeddingSuffix() )
        content = re.sub( r_re, '', content )        
        self.setRawNeoPortalContent( content )
        
        try:
            container = self.getInlineCommentContainer()
            del container[id]
        except:
            pass      

    security.declarePrivate( '_addInlineComment' )
    def _addInlineComment( self, comment ):
        """
            Add the comment to the container
        """

        container = self.getInlineCommentContainer()
        
        container[ comment.getId() ] = comment
        
    security.declarePublic( 'getInlineComment' )
    def getInlineComment( self, id ):
        """
            Return the requested comment from the container
        """

        container = self.getInlineCommentContainer()
        
        if container.has_key( id ): 
            # wrap it up to give context
            return container[id].__of__( self )

        return None

    security.declarePublic( 'getRenderedNeoPortalContent' )
    def getRenderedNeoPortalContent( self, view_comment=None, REQUEST=None, **kw ):
        """
            Return the content after rendering: first check to see if the content is cached
        """

        if REQUEST is None:
            REQUEST = self.REQUEST

        if view_comment is None:
            if REQUEST is not None:
                view_comment = REQUEST.get( 'view_comment', '1' )
            else:
                view_comment = '1'
        
        # view_comment: 1, 0, or Creator()

        if not getattr( aq_base( self ), 'neoportal_inline_commentable_content', 0 ):
            self.np_log( message="not inline commentable content/returning cached content", summary='NP_InlineCommentableContent', severity='DEBUG')
            return self.cooked_text
                
        if str( view_comment ) == '1' and self.isContentLanguageAcceptable( REQUEST ):
            self.np_log( message="inline comments enabled and user language acceptable(%s - %s)/returning cached content" % ( self.getNeoPortalUserAcceptLanguage( REQUEST ), self.getNeoPortalContentLanguage() ), summary='NP_InlineCommentableContent', severity='DEBUG')
            return self.cooked_text

        self.np_log( message="view selective comment: %s" % view_comment, summary='NP_InlineCommentableContent', severity='DEBUG')
        
        return self._renderNeoPortalContent( creator=view_comment, REQUEST=REQUEST )

    security.declarePublic( 'getRenderedInlineCommentContent' )
    def getRenderedInlineCommentContent( self, comment ):
        """
            Return the requested comment's rendered content
        """
 
        return self._cookContent( comment.getRawNeoPortalContent() )

    security.declarePublic( 'getInlineComments' )
    def getInlineComments( self ):
        """
            Return all comments from the container
        """

        return [ comment.__of__( self) for comment in self.getInlineCommentContainer().values() ]


    security.declarePublic( 'getLostInlineCommentMarkers' )
    def getLostInlineCommentMarkers( self ):
        """
            Return all lost inline comment markers
        """        
        if hasattr(self, '_np_lost_inline_comment_markers' ):
            return self._np_lost_inline_comment_markers

        else: 
            return []
        
    security.declarePublic('hasInlineComments')
    def hasInlineComments(self):
        """
            Tell if content has comments
        """                     
        return len(self.getInlineComments()) > 0
        
    security.declarePublic('getNumInlineComments')
    def getNumInlineComments(self):
        """
            Return total number of inline comments
        """                     
        
        return len( self.getInlineComments() ) 
                

    security.declarePublic( 'getInlineCommentCreators' )
    def getInlineCommentCreators( self ):
        """
            Return list of comment creators
        """
        seen={} # eliminate duplicates
        creators=[]
        for creator in [ comment.Creator() for comment in self.getInlineComments() ]:
            if seen.has_key( creator ): continue
            seen[creator]=1
            creators.append( creator)

        return creators    
        
    security.declarePublic('getInlineCommentsSearchText')
    def getInlineCommentsSearchText(self):
        """
            Return comment search text for catalogging
        """
        text = ' '.join( [ comment.getRenderedNeoPortalContent() for comment in self.getInlineComments() ] )
               
        return removeTags( text )        

    security.declarePrivate( '_renderInlineComments' )
    def _renderInlineComments( self, content, creator=None, REQUEST=None, **kw ):
        """
            Render embedded inline comments/comment form markers
        """        
        
        if not self.inlineCommentsAllowed() or creator =='0': return self._stripInlineCommentMarkers( content )
        
        if REQUEST is None:

            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
            else:
                return content 
            
        e_pattern = re.compile( self._getInlineCommentMarkerRegEx(), re.IGNORECASE)
        
        pre = len( self._getEmbeddingPrefix())
        suf = len( self._getEmbeddingSuffix())
        e_ids = [elem.strip()[pre:-suf] for elem in e_pattern.findall( content )]
        self.np_log( message="comment markers found: %s" % e_ids, summary='NP_InlineCommentableContent', severity='DEBUG')
        p_index = 1

        for e_id in e_ids:
        
            if e_id == self.getId():
                continue            # ignore self, why would you want to do that? well, Mulder says trust no one.
            
            e_re = re.escape( self._getEmbeddingPrefix()) + e_id + re.escape( self._getEmbeddingSuffix())

            # push in paragraph index
            REQUEST['p_index'] = p_index
            if e_id.startswith( self.getInlineCommentFormMarker() ):  # it's a comment form marker
            
                try:
                    c_prefix, c_idx = e_id.split('_') # comment_1, comment_2, etc
                except:
                    self.np_log( message="_renderInlineComments (bad id error): %s" % e_id, summary='NP_InlineCommentableContent' , severity='ERROR')
                    continue
                
                p_index = p_index + 1
                
                view = self.getInlineCommentButtonView()
                view = apply( view, ( self, REQUEST ) )

                content = re.sub( e_re, view, content )
                continue

            if self.getInlineComment( e_id ) is None: 
                # comment must have been deleted
                content = re.sub( e_re, '', content )                
            else:
                try:
                    e_obj = self.getInlineComment( e_id )
                    if hasattr( e_obj, '_is_temp_comment'): continue

                    if creator is not None and e_obj.Creator() != creator:
                        self.np_log( message="comment found but not selected for view (creator:%s != %s): %s" % ( e_obj.Creator(), creator, e_obj.getId() ), summary='NP_InlineCommentableContent' , severity='DEBUG')
                        content = re.sub(e_re, '', content)

                    self.np_log( message="comment found: %s" % e_obj.getId(), summary='NP_InlineCommentableContent' , severity='DEBUG')
                    e_obj = e_obj.__of__( self)
                    view = e_obj.getInlineCommentView()
                    view = apply( view, ( e_obj, REQUEST ) )
                    content = re.sub( e_re, view, content )

                except Exception, e:
                    self.np_log( message="_renderInlineComments (fatal error rendering comment): %s" % e, summary='NP_InlineCommentableContent' , severity='ERROR')
                    pass

        # render comments that lost context
        
        self._np_lost_inline_comment_markers = []
        
        lost_comments = [ comment for comment in self.getInlineComments() if comment.getId() not in e_ids ]
        for comment in lost_comments:
            if creator is not None and comment.Creator() != creator: 
                continue
            try:
                e_obj = comment.__of__( self )
                view = e_obj.getLostInlineCommentView() 
                view = apply( view, ( e_obj, REQUEST ) )
                content = content + view
            
                self._np_lost_inline_comment_markers.append( self._getEmbeddingPrefix() + comment.getId() + self._getEmbeddingSuffix() )
            
            except Exception, e:
                self.np_log( message="_renderInlineComments( fatal error rendering lost comment ): %s" % e, summary='NP_InlineCommentableContent' , severity='ERROR')
            
            self.np_log( message="comments that lost context: %s" % self.getLostInlineCommentMarkers(), summary='NP_InlineCommentableContent', severity='DEBUG')
            
        return content

NP_INLINE_COMMENT_OBJECT_PREFIX = NeoPortalInlineCommentableContent._np_inline_comment_object_prefix
                
InitializeClass( NeoPortalInlineCommentableContent )
