#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_Mailer.py,v 1.1.1.1 2002/12/13 01:56:51 wankyu Exp $
Description

    NeoPortal Mailer is a fully functional MIME mailer that can handle UNICODE

"""

from Globals import InitializeClass

from Products.NeoPortalLibrary.NP_LoggableObject import NeoPortalLoggableObject
from Products.NeoPortalLibrary.email.Header import Header

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from AccessControl import ClassSecurityInfo
import os, sys, smtplib, MimeWriter, mimetools, cStringIO, types, mimetypes


class NeoPortalMailer( NeoPortalLoggableObject ):
    """
        Simple Wrapper Class for MailHost
    """
    
    security = ClassSecurityInfo()
    
    
    def __init__( self
                , mailhost=None
                , from_addr=''
                , to_addr=''
                , reply_addr=''
                , subject=''
                , description=''
                , body=''
                , cc_addr=[]
                , bcc_addr=[]               
                , attachments=None
                , html=0
                , charset='us-ascii'
                , encoding='base64' ):
        """
            Set up mail
        """
        self.mailhost = mailhost
        self.charset= charset
                            
        self.from_addr = Header( unicode( from_addr, charset, 'replace'), self.charset ).encode()
        
        if type( to_addr ) == type ( [] ):
            self.to_addr = ';'.join( to_addr )
        else:
            self.to_addr = to_addr
            
        if type( cc_addr ) == type ( [] ):
            self.cc_addr = ';'.join( cc_addr )
        else:
            self.cc_addr = cc_addr
            
        if type( bcc_addr ) == type ( [] ):
            self.bcc_addr = ';'.join( bcc_addr )
        else:
            self.bcc_addr = bcc_addr
            
        self.reply_addr = Header( unicode( reply_addr, charset, 'replace' ), self.charset ).encode()
        self.subject = Header( unicode( subject, charset, 'replace') , self.charset ).encode()
        self.description = description # plain-text description of the message (html handling info, etc)
        self.body = body # plain-text or html body
        self.attachments = attachments
        self.is_html = html
        self.encoding = encoding
        self.cc_addr = self.bcc_addr = None

    security.declarePrivate( 'send' )
    def send( self ):
        """
            Send mail
        """
        
        message = cStringIO.StringIO()
        
        body_in = cStringIO.StringIO( self.body )
        desc_in = cStringIO.StringIO( self.description )
        
        writer = MimeWriter.MimeWriter( message )
        writer.addheader( 'MIME-Version', '1.0' )
        writer.addheader( 'From', self.from_addr )
        writer.addheader( 'To', self.to_addr )
        
        writer.addheader( 'Reply-to', self.reply_addr )
        
        if self.cc_addr is not None:
            writer.addheader( 'CC', self.cc_addr )

        if self.bcc_addr is not None:
            writer.addheader( 'BCC', self.bcc_addr )
        
        writer.addheader( 'Subject', self.subject )
        writer.startmultipartbody( "mixed" )
        writer.flushheaders()

        # plain text description
        
        if self.description:
            part = writer.nextpart()
            part.addheader( "Content-Transfer-Encoding", self.encoding )
            pout = part.startbody( "text/plain", [ ( "charset", self.charset ) ] )
            mimetools.encode( desc_in, pout, self.encoding )
            desc_in.close()
        
        # plain text or html body
        
        part = writer.nextpart()
        part.addheader( "Content-Transfer-Encoding", self.encoding )
        
        if self.is_html:
            pout = part.startbody( 'text/html', [ ( "charset", self.charset ) ] )
        else:
            pout = part.startbody( 'text/plain', [ ( "charset", self.charset ) ] )
        
        mimetools.encode(body_in, pout, self.encoding )
        body_in.close()
    
        if self.attachments is not None:
            for file in self.attachments:
                part = writer.nextpart()
                part.addheader( 'Content-Transfer-Encoding', self.encoding )
                
                if hasattr( file, 'content_type' ) and hasattr( file, 'content'):
                    if hasattr( file, 'filename'):
                        part.addheader( 'Content-Disposition', 'attachment; filename=%s' % Header( unicode( filename, self.charset, 'replace' ), self.charset ).encode() )
                    pout = part.startbody( file['content_type'], [ ( "charset", self.charset ) ] )
                    mimetools.encode( file['content'] , pout, self.encoding )
                else:
                    if type( file ) is types.StringType: 
                        part.addheader( 'Content-Disposition', 'attachment; filename=%s' % Header( unicode( os.path.basename( file ), self.charset, 'replace' ), self.charset ).encode() )                    
                        instream = open( file, 'rb' )
                        close_in = 1
                        content_type, dummy = mimetypes.guess_type( file )                    
                    else:
                        instream = file
                        close_in = 0
                        content_type = 'application/octet-stream'
                    
                    pout = part.startbody( content_type, [ ( "charset", self.charset ) ] )
                    mimetools.encode( instream , pout, self.encoding )

                    if close_in:
                        instream.close()
    
        writer.lastpart()
        
        self.np_log( message="Sending mail from %s to %s (reply: %s ): %s" % ( self.from_addr, self.to_addr, self.reply_addr, self.subject ), summary='NP_Mailer', severity='INFO')
    
        outmsg = message.getvalue()
        if not self.mailhost:
            import smtplib
            server = smtplib.SMTP("localhost")
            server.sendmail( self.from_addr, self.to_addr, outmsg )
            server.quit()
        else:
            self.mailhost.send( outmsg )

        message.close()        

InitializeClass( NeoPortalMailer )
