#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_QuotaAwareObject.py,v 1.1 2003/01/10 16:52:12 wankyu Exp $
Description
    
    NeoPortal Quota Aware Object
    
"""

from Globals import InitializeClass
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner, aq_chain
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalQuotaAwareObject( NeoPortalBaseItem ): 
    """ 
        NeoPortal Quota Aware Object
    """

    neoportal_quota_aware = 1
    
    security = ClassSecurityInfo()

    security.declarePrivate( 'getNeoPortalQuotaContainer' )
    def getNeoPortalQuotaContainer( self ):
        """
            Return Quota Container
        """

        for container in aq_chain( self ):
            if hasattr( aq_base( container ), 'isNeoPortalContainerQuotaEnabled' ) and container.isNeoPortalContainerQuotaEnabled():        
                return container
                
        return None

    security.declarePrivate( '_checkNeoPortalContainerQuota' )        
    def _checkNeoPortalContainerQuota( self ):        
        """
            Update Quota Info
        """
        
        quota_container = self.getNeoPortalQuotaContainer()
        if quota_container is not None:
            quota_container._checkNeoPortalContainerQuota( self )

        # okay? update info
        self._updateNeoPortalQuotaContainerInfo()
            
    security.declarePrivate( '_updateNeoPortalQuotaContainerInfo' )        
    def _updateNeoPortalQuotaContainerInfo( self ):        
        """
            Update Quota Info
        """
        
        quota_container = self.getNeoPortalQuotaContainer()
        if quota_container is not None:
            quota_container._recalculateNeoPortalContainerQuota()
        
InitializeClass( NeoPortalQuotaAwareObject )