#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_RatableContent.py,v 1.6 2003/03/30 13:07:31 wankyu Exp $
Description

    NeoPortal Ratable Content can be rated by users
 
"""

from Globals import InitializeClass
from BTrees.OIBTree import OIBTree
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_AccessTraceableObject import NeoPortalAccessTraceableObject

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalRatableContent( NeoPortalAccessTraceableObject ):
    """
        Mix-in that allows content rating
    """
    
    neoportal_ratable_content = 1
    _np_min_content_rating_value = 1
    _np_max_content_rating_value = 10
    
    _np_content_rating_value_top_limit = 100    # inclusive
    _np_content_rating_value_bottom_limit = 0   # inclusive
    
    _np_content_rating_container_name = 'neoportal_content_ratings'
    
    security = ClassSecurityInfo()    

    security.declarePublic( 'contentRatingAllowed' )        
    def contentRatingAllowed( self ):
        """
           Tell if content rating is allowed
        """

        if not getattr( self, 'neoportal_ratable_content', 0 ): return 0
        
        if hasattr( self, 'np_content_rating_allowed' ):
            return self.np_content_rating_allowed
        else:
            try:
                return self.neoportal_properties.isContentRatable( self )
            except:
                return 0
                
    security.declareProtected( NeoPortalModifyPortalContent, 'getNeoPortalContentRatingTraceKey' )        
    def getNeoPortalContentRatingTraceKey( self ):
        """
            Return how to trace a user access
        """

        return self.getNeoPortalAccessTraceKey( container_name=self._np_content_rating_container_name )
            
    security.declarePublic( 'getMinContentRatingValue' )        
    def getMinContentRatingValue( self ):
        """
           Return minimum content rating value
        """
        if hasattr( self, 'np_min_content_rating_value' ):
            return int( self.np_min_content_rating_value )
        else:
            try:
                return int( self.neoportal_properties.min_content_rating_value )
            except:
                return self._np_min_content_rating_value
                

    security.declarePublic( 'getMaxContentRatingValue' )        
    def getMaxContentRatingValue( self ):
        """
           Return maximum content rating value
        """
        if hasattr( self, 'np_max_content_rating_value' ):
            return int( self.np_max_content_rating_value )
        else:
            try:
                return int( self.neoportal_properties.max_content_rating_value )
            except:
                return self._np_max_content_rating_value 
                
    security.declarePublic( 'getNeoPortalContentRatingContainerURL' )    
    def getNeoPortalContentRatingContainerURL( self ):
        """
           Return this object's absolute url
        """
        return self.absolute_url()
                    
    security.declarePrivate( 'getNeoPortalContentRatingContainer' )     
    def getNeoPortalContentRatingContainer( self ):
        """
            Return container that hold ratings
        """

        return self.getNeoPortalAccessTraceContainer( container_name=self._np_content_rating_container_name  )
        
    security.declarePublic( 'addNeoPortalContentRating' )
    def addNeoPortalContentRating( self, rating, REQUEST=None ):
        """
            Add Content Rating
        """        
        
        if not self.checkNeoPortalContentReviewStatus():
            if REQUEST is not None:
                ret_url = REQUEST.get('ret_url', self.absolute_url())
                message = self.getNeoPortalMessage( message="You can't rate unpublished content." ) 
                return REQUEST['RESPONSE'].redirect(
                    ret_url
                    + "?portal_status_message=%s" % message
                    ) 
            return
                
        try:
            rating = int(rating)
        except:
            if REQUEST is not None:
                ret_url = REQUEST.get('ret_url', self.absolute_url())
                message = self.getNeoPortalMessage( message="Invalid rating value." )
                return REQUEST['RESPONSE'].redirect(
                    ret_url
                    + "?portal_status_message=%s" % message
                    )           
            else:
                return
        
        if rating < self.getMinContentRatingValue():
            rating = self.getMinContentRatingValue()
        if rating > self.getMaxContentRatingValue():
            rating = self.getMaxContentRatingValue()

        prev_rating = self.getNeoPortalAccessTraceValue( container_name=self._np_content_rating_container_name, REQUEST=REQUEST )
        self.addNeoPortalAccessTrace( trace_value=rating, container_name=self._np_content_rating_container_name, REQUEST=REQUEST )
        
        if REQUEST is not None:
            if prev_rating is None:
                message = self.getNeoPortalMessage( message="You gave %d to this content.", tup=( rating, ) )
            elif prev_rating == rating:
                message = self.getNeoPortalMessage( message="Your rating remains the same: %d.", tup=( rating, ) )
            else:
                message = self.getNeoPortalMessage( message="Your rating has changed from %d to %d.", tup=( prev_rating, rating ) )

            ret_url = REQUEST.get('ret_url', self.absolute_url())
            
            return REQUEST['RESPONSE'].redirect(
                ret_url
                + "?portal_status_message=%s" % message
                )           

    security.declarePublic( 'getNeoPortalUserContentRating' )        
    def getNeoPortalUserContentRating( self, date=None, ip_address=None, username=None, REQUEST=None ):
        """
            Return rating value of the current user
        """

        return self.getNeoPortalAccessTraceValue( container_name=self._np_content_rating_container_name, date=date, ip_address=ip_address, username=username, REQUEST=REQUEST )

    security.declareProtected( NeoPortalModifyPortalContent, 'deleteNeoPortalContentRating' )        
    def deleteNeoPortalContentRating( self, date=None, ip_address=None, username=None, REQUEST=None ):
        """
            Delete Content Rating
        """

        result = self.deleteNeoPortalAccessTrace( container_name=self._np_content_rating_container_name, date=date, ip_address=ip_address, username=usename, REQUEST=REQUEST )

        if REQUEST is not None:
            if result is not None:
                message = self.getNeoPortalMessage( message=result )
            else:
                message = self.getNeoPortalMessage( message="Deleted." )
                            
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=%s" % message
                )                       
                
    security.declareProtected( NeoPortalModifyPortalContent, 'initNeoPortalContentRatings' )        
    def initNeoPortalContentRatings( self, REQUEST=None ):
        """
            Initialize the container
        """
        
        self.initNeoPortalAccessTraceContainer( container_name=self._np_content_rating_container_name )
            
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message="Container initialized." )
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=%s" % message
                )                


    security.declarePublic( 'getNeoPortalNumContentRatings' )
    def getNeoPortalNumContentRatings( self ):
        """
           Return the number of content ratings
        """
        self.np_num_ratings = self.getNeoPortalNumAccessTraces( container_name=self._np_content_rating_container_name )

        return self.np_num_ratings
        
    security.declarePublic( 'getNeoPortalAverageContentRating' )    
    def getNeoPortalAverageContentRating( self ):
        """
           Return the average content rating
        """
        ratings = self.getNeoPortalContentRatingContainer().values()
        if not ratings:
            return 0
            
        return round( float( reduce( lambda x,y: x+y, ratings ) ) / float( self.getNeoPortalNumContentRatings() ), 1 )
            
InitializeClass( NeoPortalRatableContent )

