#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_STDocument.py,v 1.10 2003/04/02 06:42:24 wankyu Exp $
Description

    NeoPortal Structured Text defines a set of new ST rules.
    
    Mostly... for personal uses;-)
 
"""
from StructuredText import HTMLClass, HTMLWithImages, DocumentClass, DocumentWithImages, STDOM, ST
from StructuredText.DocumentClass import StructuredTextUnderline, StructuredTextSection as BaseTextSection

from Products.NeoPortalLibrary.NP_STletters import e_letters, letters, digits, literal_punc, under_punc, strongem_punc, phrase_delimiters,dbl_quoted_punc

import re

BaseDocumentClass = DocumentWithImages.__class__
BaseHTMLClass = HTMLWithImages.__class__

NEOPORTAL_PARAGRAPH_MARKER = '\t{::NEOPORTAL_PARAGRAPH_MARKER::}\t'

class NeoPortalHTMLClass( BaseHTMLClass ):

    def paragraph( self, doc, level, output ):

        if hasattr(doc, 'css_class' ):
            if doc.css_class != 'np_ignorable_paragraph':
                output('<p class="' + doc.css_class + '">')
            else:
                output('<p>')

            for c in doc.getChildNodes( ):
                if c.getNodeName() in ['StructuredTextParagraph']:
                    getattr( self, self.element_types[c.getNodeName()])(
                        c, level, output)
                else:
                    getattr( self, self.element_types[c.getNodeName()])(
                        c, level, output)

            output('</p>\n\n')                    
                
        else:
            
            output('<p>')
            
            for c in doc.getChildNodes( ):
                if c.getNodeName() in ['StructuredTextParagraph']:
                    getattr( self, self.element_types[c.getNodeName()])(
                        c, level, output)
                else:
                    getattr( self, self.element_types[c.getNodeName()])(
                        c, level, output)

            output('</p>\n\n')

    def section( self, doc, level, output ):

        children=doc.getChildNodes()
        idx = 0
        for c in children:        
            getattr( self, self.element_types[c.getNodeName()])(c, level+1, output)
        
            if hasattr(doc, 'css_class') and idx==0:
                output('<div class="' + doc.css_class + '">\n')        
                idx=idx+1
        
        if hasattr(doc, 'css_class' ):
            output('</div>\n')

class StructuredTextParagraph( ST.StructuredTextParagraph ):
    """
        ST Paragraph that can squeeze in a css class
    """
    
    indent=0

    def __init__( self, src,    subs=None, css_class=None, is_section=0, **kw ):
        if subs is None: subs=[]

        self._src=src        
        self._subs=list(subs)
        if css_class:
            self.css_class = css_class

        self.is_section = is_section
        self._attributes=kw.keys()
        
        for k, v in kw.items():
            setattr( self, k, v )

class StructuredTextSection( StructuredTextParagraph ):
    """
        ST Section that can squeeze in a css class
    """
        
    def __init__( self, src,    subs=None, header_css_class='', body_css_class='', **kw ):
       apply( StructuredTextParagraph.__init__
            , ( self,    StructuredTextParagraph( src, css_class=header_css_class ), subs, body_css_class, 1 )
            , kw )

class NeoPortalSTDocument( BaseDocumentClass ):
    """
        NeoPortal Structured Text Document
    """
    
    paragraph_types  = [
        'doc_example',
        'doc_translation', 
        ] + BaseDocumentClass.paragraph_types
    
    text_types = [
        'doc_named_link',
        'doc_inner_link',
        ] + BaseDocumentClass.text_types
            
    def doc_bullet( self, paragraph,    expr = re.compile( r'\s*[-*o]\s+' ).match ):
        """
            Regex hack
        """
        return BaseDocumentClass.doc_bullet( self, paragraph, expr )

    def doc_numbered( self, paragraph, expr = re.compile( r'(\s*[0-9]+\.)|(\s*[0-9]+\s+)' ).match ):
        """
            Regex hack: disabled
        """
        
        return None
        
        #return BaseDocumentClass.doc_numbered( self, paragraph, expr )        

    def doc_description( self, paragraph, delim = re.compile( r'\s+--\s+' ).search, nb=re.compile( r'[^\000- ]' ).search, ):
        """
            Regex hack
        """
        return BaseDocumentClass.doc_description( self, paragraph, delim, nb )        

    def doc_header( self, paragraph ):
        """
            Add a couple of new header notations
        """
        
        subs=paragraph.getSubparagraphs()
        if not subs: return None
        top=paragraph.getColorizableTexts()[0].strip()
        if not top.strip(): return None
        if top[-2:]=='::':
           subs=DocumentClass.StructuredTextExample(subs)
           if top.strip() =='::': return subs
           # copy attrs when returning a paragraph
           kw = {}
           atts = getattr(paragraph, '_attributes', [])
           for att in atts: kw[att] = getattr(paragraph, att)
           return apply(ST.StructuredTextParagraph, (top[:-1], [subs]), kw)
        elif top[-2:]==';;':
            return StructuredTextSection(top[:-2], subs, header_css_class='np_term_header', body_css_class='np_term_body', indent=paragraph.indent)
        elif top[-2:]=='##':
            return StructuredTextSection(top[:-2], subs, header_css_class='np_ignorable_paragraph', body_css_class='np_long_example', indent=paragraph.indent)
        elif top[-2:]=='$$':
            return StructuredTextSection(top[:-2], subs, header_css_class='np_ignorable_paragraph', body_css_class='np_long_translation', indent=paragraph.indent)
        elif top[-2:]=='!!':
            return StructuredTextSection(top[:-2], subs, header_css_class='np_ignorable_paragraph', body_css_class='np_long_keywords', indent=paragraph.indent)
           
        if top.find('\n') >= 0: return None
        return BaseTextSection( top, subs, indent=paragraph.indent )

    def doc_literal( self, s, expr = re.compile( r"(\W+|^)'([^']+)'([%s]+|$)" % ( phrase_delimiters,) ).search, ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_literal( self, s, expr )
    
    def doc_inner_link( self, s, expr1 = re.compile( r"\.\.\s*" ).search, expr2 = re.compile( r"\[%s]+\]" % ( digits, ) ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_inner_link( self, s, expr1, expr2 )
        
    
    def doc_named_link( self, s, expr=re.compile( r"(\.\.\s)(\[[%s]+\])" % ( digits, ) ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_inner_link( self, s, expr )
    
    def doc_underline( self, s, expr = re.compile( r"(\W+|^)_([^_]+)_(\W+|$)" ).search, ):
        """
            Skip NeoPortal Object Embedding notations
        """        
        result = expr(s)
        if result:
            start,end = result.span(2)
            st,e = result.span()
            return ( StructuredTextUnderline( s[start:end]) , st+len( result.group(1) ) , e-len( result.group(3) ) )
        else:
            return None

    def doc_emphasize( self, s, expr = re.compile( r'\*([%s%s%s\s]+?)\*' % ( letters, digits, strongem_punc ) ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_emphasize( self, s, expr )
    
    def doc_strong( self, s, expr = re.compile( r'\*\*([%s%s%s\s]+?)\*\*' % ( letters, digits, strongem_punc ) ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_strong( self, s, expr )

    ## Some constants to make the doc_href() regex easier to read.
    _DQUOTEDTEXT = r'("[^\"]+")'## double quoted text
    _ABSOLUTE_URL=r'((http|https|ftp|mailto|file|about)[:/]+?[%s0-9_\@\.\,\?\!\/\:\;\-\#\~\=\&\%%\+]+)' % e_letters
    _ABS_AND_RELATIVE_URL=r'([%s0-9_\@\.\,\?\!\/\:\;\-\#\~\=\&\%%\+]+)' % e_letters
    _URL_AND_PUNC = r'([%s0-9_\@\.\,\?\=\&\+\!\/\:\;\-\#\~]+)'  % e_letters

    _SPACES = r'(\s*)'

    def doc_href1( self, s, expr=re.compile( _DQUOTEDTEXT + "(:)" + _ABS_AND_RELATIVE_URL + _SPACES ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_href1( self, s, expr )

    def doc_href2( self, s, expr=re.compile( _DQUOTEDTEXT + r'(\,\s+)' + _ABSOLUTE_URL + _SPACES).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_href2( self, s, expr )         

    def doc_href( self, s
                , expr = re.compile( _DQUOTEDTEXT
                                    + "(:)"
                                    + _URL_AND_PUNC
                                    + _SPACES
                                    ).search
                , punctuation = re.compile( _DQUOTEDTEXT
                                    + r'(\,\s+)'
                                    + _URL_AND_PUNC
                                    + _SPACES
                                    ).search
                ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_href( self, s, expr, punctuation )         
    
    def doc_sgml( self,s,expr=re.compile(r"\<[%s0-9\.\=\'\"\:\/\-\#\+\s\*]+\>" % e_letters).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_sgml( self, s, expr )

    def doc_xref( self, s, expr = re.compile( '\[([0-9\-.:/;,\n\r\~]+)\]' ).search ):
        """
            Regex hack
        """        
        return BaseDocumentClass.doc_xref( self, s, expr )

    def doc_example( self, paragraph, expr = re.compile( r'\s*[#]\s+' ).match ):
        """
            Example paragraph
        """
        top=paragraph.getColorizableTexts()[0]
        m=expr(top)

        if not m:
            return None
            
        subs=paragraph.getSubparagraphs()

        return StructuredTextParagraph( top[m.span()[1]:], subs, css_class="np_example", indent=paragraph.indent )

    def doc_translation( self, paragraph, expr =    re.compile(r'\s*[$]\s+').match ):
        """
            Translation Paragraph
        """
        top=paragraph.getColorizableTexts()[0]
        m=expr(top)

        if not m:
            return None
            
        subs=paragraph.getSubparagraphs()

        return StructuredTextParagraph( top[m.span()[1]:], subs,css_class="np_translation", indent=paragraph.indent )

Document = NeoPortalSTDocument()
HTMLNG = NeoPortalHTMLClass()

def NeoPortalStructuredDocumentHTMLNG( content ):
    """
        Render stx_doc into HTML and return it
    """
        
    stx_doc = ST.Basic( content )
    
    stx_doc = Document( stx_doc )

    return HTMLNG( stx_doc, header=0 )
