#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_SpamBlockableObject.py,v 1.2 2003/01/10 16:51:45 wankyu Exp $
Description

    NeoPortal Spam Bloackable Object can block spamming (DOS attacks) 
        by keeping track of content posters' IP's and the time of their postings
 
"""

from Globals import InitializeClass
from Globals import PersistentMapping
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalSpamBlockableObject( NeoPortalBaseItem ):
    """
        Mix-in that allows spam blocking
    """
    
    neoportal_spam_blockable_content = 1
    
    _np_spam_blocking_interval    = 10    # interval in seconds to allow duplicate postings from the same IP
    _np_max_num_spam_blocking_log_entries    = 20    # maximum num of entries to store in the log 
    
    security = ClassSecurityInfo()    
    
    security.declarePublic( 'accessLoggingEnabled' )        
    def spamBlockingEnabled( self ):
        """
           Tell if spam blocking is enabled
        """
        
        if not getattr( self, 'neoportal_spam_blockable_content', 0 ): return 0
        
        if hasattr( self, 'np_spam_blocking_enabled' ):
            return self.np_spam_blocking_enabled
        
        try:
            return self.neoportal_properties.isContentSpamBlockable( self )
        except:
            return self.neoportal_spam_blockable_content
                    
    security.declarePublic( 'getSpamBlockingInterval' )        
    def getSpamBlockingInterval( self ):
        """
           Return interval for spam blocking
        """
        
        if hasattr( self, 'np_spam_blocking_interval' ):
            return self.np_spam_blocking_interval        
        
        try:
            return self.neoportal_properties.spam_blocking_interval
        except:
            return self._np_spam_blocking_interval

    security.declarePublic( 'getMaxNumSpamBlockingLogEntries' )        
    def getMaxNumSpamBlockingLogEntries( self ):
        """
           Return max number of entries in the log
        """
        
        if hasattr( self, 'np_max_num_spam_blocking_log_entries' ):
            return self.np_max_num_spam_blocking_log_entries        
        
        try:
            return self.neoportal_properties.max_num_spam_blocking_log_entries
        except:
            return self._np_max_num_spam_blocking_log_entries
                    
    security.declarePrivate( 'getPostingLogContainer' )          
    def getPostingLogContainer( self ):
        """
            Return container that hold info on recent content posters
        """
        
        if not hasattr( self, '_np_content_posting_log' ):
            self._np_content_posting_log = PersistentMapping()
        
        return self._np_content_posting_log

    getContentPostingLog = getPostingLogContainer
    
    security.declareProtected( NeoPortalModifyPortalContent, 'getLastestContentPosterIPs' )          
    def getLastestContentPosterIPs( self ):
        """
            Return latest posters' IP addresses
        """
        return self.getPostingLogContainer().keys()
    
    security.declareProtected( NeoPortalModifyPortalContent, 'getContentPostingTime' )         
    def getContentPostingTime( self, ip_address=None, REQUEST=None ):
        """
            Return posting time
        """     
        
        if ip_address is None:           
            ip_address = self._getRemoteIP(REQUEST=REQUEST)

        try:
            container = self.getPostingLogContainer()
            return container[ip_address]
        except:
            # no posting log found; gotta be a new posting
            return 0

    security.declareProtected( NeoPortalModifyPortalContent, 'setContentPostingTime' )
    def setContentPostingTime( self, ip_address=None, REQUEST=None ):
        """
            Return posting time
        """     

        if ip_address is None:           
            ip_address = self._getRemoteIP(REQUEST=REQUEST)
        try:
            
            max_num = self.getMaxNumSpamBlockingLogEntries()
            container = self.getPostingLogContainer()
            
            if len(container) > max_num:
                entries = container.keys()
                entries.sort()
                entries.reverse()
                first = entries.pop()

                del container[first]
            
            container[ip_address] = int(DateTime().timeTime())

        except:
            pass

    security.declareProtected( NeoPortalModifyPortalContent, 'validateDuplicateNeoPortalContentPosting' )
    def validateDuplicateNeoPortalContentPosting( self, REQUEST=None ):
        """
            Tell whether to allow a new posting from a remote IP
        """     
        
        if not self.spamBlockingEnabled(): return 1
        
        if REQUEST is None:
            REQUEST = self.REQUEST

        if REQUEST is None:
            return 1
        
        ip_address = self._getRemoteIP( REQUEST=REQUEST )

        latest_posting_time = self.getContentPostingTime( ip_address=ip_address )

        
        if ( int( DateTime().timeTime() ) - int( latest_posting_time ) ) <= int( self.getSpamBlockingInterval() ):
            self.np_log( message="New posting: %s / Last posting: %s (%s)" % (int(DateTime().timeTime()), int(latest_posting_time), ip_address ), summary='NP_SpamBlockableObject', severity='DEBUG')
            message = self.getNeoPortalMessage( message="Please wait for %s second(s) to post new content.", tup=( self.getSpamBlockingInterval(), ) )            
            raise 'NeoPortalError', message

        self.setContentPostingTime( REQUEST=REQUEST )
            
    security.declareProtected( NeoPortalModifyPortalContent, 'initContentPostingLog' )
    def initContentPostingLog( self, REQUEST=None ):
        """
            initialize the log
        """
        
        self.np_content_posting_log = PersistentMapping()
            
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + "?portal_status_message=Log initialized."
                )
            
InitializeClass( NeoPortalSpamBlockableObject )

