#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_TranslatableContent.py,v 1.4 2003/01/12 17:55:32 wankyu Exp $
Description

    NeoPortal Translatable Content eases up i18n process of internal user/error messages. 
    
    All internal user/error messages should be directed to:
    
        - getNeoPortalMessage ( message='', tup=() ) : message translation

        - manageNeoPortalUserMessage( REQUEST=None, message='', tup=()): user message manager

        - manageNeoPortalError( obj=None, REQUEST=None, field='', message='', tup=() ): handle error
 
"""

from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_Utils import setFormMsg, setFormError

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import re, os

class NeoPortalTranslatableContent( NeoPortalBaseItem ):
    """
        Mix-in that helps i18n of internal messages
    """
    
    neoportal_translatable_content = 1
    
    _default_domain = 'neoportal'
    
    security = ClassSecurityInfo()
    
    security.declarePublic( 'isContentTranslatable' )
    def isContentTranslatable( self ):
        """
            Tell if content is translatable
        """
        
        if not getattr( self, 'neoportal_translatable_content', 0 ): return 0
        if getattr( self, 'neoportal_cacheable_content', 0 ) and not self.isContentCacheable(): return 0
        
        return 1    

    security.declarePrivate( '_getTranslationService' )
    def _getTranslationService( self ):
        """
            Return translation_service instance if found
        """
        
        return getattr( self, 'translation_service', None)

    
    security.declarePublic( 'getNeoPortalMessage' )
    def getNeoPortalMessage ( self, message='', tup=() ):
        """
            Return translated message: not so sophisticated
        """
        
        tup = tuple( tup )
        ts = self._getTranslationService()
        org_message = message % tup

        # supports only primitive formats to prevent unnecessary complexity
        # supported: %d, %10d, %0.5f style formatting
        # not supported: %(x)-10d style complex format strings;
        
        format_re = r'%[\*\.\-0-9]*?[diuxXofegGsc]' 
        f_pattern = re.compile( format_re )
        
        # suppose: message = '%d articles deleted from %d boards by the user %s'
        
        format_strings = [ f.strip() for f in f_pattern.findall( message ) ]    
        
        # format_strings would be: ['%d', '%d', '%s']
        # tup would be: [10, 2, 'wankyu']
        
        if ts is not None:
            mapping = {}
            
            try:
                idx = 1
                for t in tup:
                    #
                    # mapping would be: {'var_1': '10', 'var_2': '2', 'var_3': 'wankyu'}
                    #
                    varname = 'var_' + str( idx )
                    message = message.replace( format_strings[idx - 1], '${%s}'  % varname , 1 )

                    mapping[varname] = str( format_strings[idx - 1] % ( t, ) )
                    idx=idx+1
                
                message = ts.translate(domain=self._default_domain, msgid=message, mapping=mapping)
                
                if not message or message is None:
                    return org_message
                    
            except Exception, e:
                self.np_log( message="getNeoPortalMessage(%s): %s (%s) - %s" % ( e, org_message, tup, format_strings ), summary='NP_TranslatableContent', severity='DEBUG')
                pass

            try:
                print message
            except UnicodeError:
                return org_message
                
            return message
        else:
            return org_message
            
    security.declarePrivate( 'manageNeoPortalUserMessage' )
    def manageNeoPortalUserMessage( self, REQUEST=None, message='', tup=()):
        """
            Override this method to handle user messages differently
        """
        
        ret_message = self.getNeoPortalMessage( message, tup )
                    
        return setFormMsg( REQUEST, ret_message )

    security.declarePrivate( 'manageNeoPortalError' )
    def manageNeoPortalError( self, obj=None, form=None, field='', message='', tup=(), REQUEST=None):
        """
            Override this method to handle fatal errors differently. 
            
                obj is a skin object to apply the message
        """
        ret_message = self.getNeoPortalMessage( message, tup )
        
        if REQUEST is None:
            REQUEST = self.REQUEST

        if form and callable( form ) and REQUEST:
            
            REQUEST.set('portal_status_message', 'Please correct the indicated errors: %s' % ret_message)

            if obj is not None:
                return apply( form, ( obj, setFormError( REQUEST, field, ret_message ), ) )
            else:
                return apply( form, ( self, setFormError( REQUEST, field, ret_message ), ) )
        else:
            self.np_log( message="manageNeoPortalError: %s (%s')" % ( field, ret_message ), summary='NP_TranslatableContent', severity='DEBUG')
        
            raise 'NeoPortalError', '%s: %s' % ( field, ret_message )        
                    
InitializeClass( NeoPortalTranslatableContent )

