#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_UpgradableObject.py,v 1.3 2002/12/14 19:25:08 wankyu Exp $
Description

    NeoPortal Upgradable Objects can be easily upgraded
   
    All upgradable objects should call 'setCurrentClassVersion( CLASSNAME.__NEOPORTAL_CLASS_VERSION__ )' in their '__init__' method.
    In order to upgrade previous instances of a NeoPortal class implement 'selfUpgradeNeoPortalContent' method.
    
    Make sure to call 'notifyNeoPortalObjectUpgradeComplete' method in the 'selfUpgradeNeoPortalContent' after the completing upgrade process.
 
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

class NeoPortalUpgradableObject( NeoPortalBaseItem ):
    """
        Mix-in class which provides methods for self-upgrading
    """
    
    neoportal_upgradable_object = 1
    
    security = ClassSecurityInfo()
    

    security.declarePublic( 'isContentUpgradable' )
    def isContentUpgradable( self ):
        """
            Tell if content is upgradable
        """
        
        if not getattr( self, 'neoportal_upgradable_object', 0 ): return 0
        
        return 1    
    
    security.declarePrivate( '_setCurrentClassVersion' )
    def _setCurrentClassVersion( self, version ):
        """
            Set the current version
        """
        
        self._np_upgraded_up_to = version
    
    security.declareProtected( NeoPortalManagePortal, 'getNeoPortalClassVersion' )
    def getNeoPortalClassVersion( self ):
        """
            Return NeoPortal Class version
        """
    
        if hasattr( self, '__NEOPORTAL_CLASS_VERSION__' ):
            return self.__NEOPORTAL_CLASS_VERSION__
        else:
            None

    security.declareProtected( NeoPortalManagePortal, 'getNeoPortalClassPreviousVersion' )
    def getNeoPortalClassPreviousVersion( self ):
        """
            Return NeoPortal Class old version 
        """
    
        if hasattr( self, '_np_upgraded_up_to' ):
            return self._np_upgraded_up_to
        else:
            return None
            
    security.declareProtected( NeoPortalManagePortal, 'isNeoPortalObjectUpgradeNeeded' )
    def isNeoPortalObjectUpgradeNeeded( self ):
        """
            Tell if new version of class is installed
        """

        self.np_log( message="Upgrade needed?: %s of %s (Prev: %s New: %s) " % ( self.meta_type, self.getId(), self.getNeoPortalClassPreviousVersion(), self.getNeoPortalClassVersion() ), summary='NP_LoggableObject', severity='DEBUG')
        return self.getNeoPortalClassVersion() != self.getNeoPortalClassPreviousVersion()

    security.declareProtected( NeoPortalManagePortal, 'notifyNeoPortalObjectUpgradeComplete' )
    def notifyNeoPortalObjectUpgradeComplete( self ):
        """
            Complete upgrade
        """
        self._np_upgraded_up_to = self.__NEOPORTAL_CLASS_VERSION__        
        self.np_log( message="Completing upgrade: %s of %s (Prev: %s New: %s) " % (self.meta_type, self.getId(), self._np_upgraded_up_to, self.__NEOPORTAL_CLASS_VERSION__), summary='NP_LoggableObject', severity='DEBUG')

    security.declareProtected( NeoPortalManagePortal, 'selfUpgradeNeoPortalContent' )
    def selfUpgradeNeoPortalContent( self, force_upgrade=0):
        """
            Upgrade Self: force_upgrade does what its name implies
        """
        
        if self.isNeoPortalObjectUpgradeNeeded():
            return "Upgrade method not implemented."
            
            #self.notifyNeoPortalObjectUpgradeComplete()
        else:
            return "<em><b> %s </b></em> of <em> %s </em> class is up to date." % ( self.getId(), self.meta_type )

    security.declareProtected( NeoPortalManagePortal, 'getNeoPortalContentUpgradeForm' )
    def getNeoPortalContentUpgradeForm( self ):
        """
            Return appropriate form depending on the state of the instance
        """
        
        if not self.isContentUpgradable():
            return 'The object <em>%s</em> of type <em>%s</em> is out of date and does not support self-upgrading.' % ( self.getId(), self.meta_type )
            
        if not self.isNeoPortalObjectUpgradeNeeded():
            return 'The object <em>%s</em> of type <em>%s</em> is already up to date.' % ( self.getId(), self.meta_type )
            
        form_str = "<html><body><div align=\"center\">"
        form_str = form_str + "<ul><li>This NeoPortal instance <em><b>%s</b></em> of <em>%s</em> needs upgrading.</li></ul>" % ( self.getId(), self.getNeoPortalClassPreviousVersion() )
        form_str = form_str + "<ul><li>Before you proceed, make sure you have backed up your data. You've been warned.</li></ul>"
        form_str = form_str + "<form action=\"" + self.absolute_url()+  "/selfUpgradeNeoPortalContent\" method=\"post\">\n"
        form_str = form_str + "<input type=\"submit\" name=\"submit\" value=\"Upgrade to version " + self.getNeoPortalClassVersion() + "\">"
        form_str = form_str + "</form>\n"
        form_str = form_str + "</div></body></html>\n"
        
        return form_str

    security.declareProtected( NeoPortalManagePortal, 'showNeoPortalInstanceProperties' )
    def showNeoPortalInstanceProperties( self ):
        """
            For easy comparison
        """

        lines = []

        for key in self.__dict__.keys():
            lines.append( key + ' : ' + str( self.__dict__[key] ) )

        return '<html><body>' + '<br />'.join( lines ) + '</body></html>'

InitializeClass( NeoPortalUpgradableObject )
