#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NP_Utils.py,v 1.6 2003/01/06 09:24:32 wankyu Exp $
Description

    NeoPortal Utilities
"""

from Globals import DTMLFile, package_home
from OFS.Image import Image
from StructuredText import ST
from Acquisition import aq_get, aq_parent, aq_base

from Products.PythonScripts.PythonScript import manage_addPythonScript
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
from StructuredText import STDOM

from Products.NeoPortalLibrary.NP_STDocument import NeoPortalSTDocument, StructuredTextParagraph, NEOPORTAL_PARAGRAPH_MARKER

import os, os.path, string, types, urllib, re

try:
    from crypt import crypt
except:
    from Products.NeoPortalLibrary.fcrypt.fcrypt import crypt

security = ModuleSecurityInfo( 'Products.NeoPortalLibrary.NP_Utils' )

security.declarePublic( 'isEmailValid', )

NP_Salt = '.v'  

def encryptNeoPortalPassword( password ):
    """
        Encrypt the given plain-text password
    """
    return crypt( password, NP_Salt )
    
def flattenSTDoc( stx_doc, append ):
    """
        Flatten ST Doc
    """
    
    if stx_doc.getSubparagraphs(): has_sub = 1
    else: has_sub = 0

    if stx_doc.getColorizableTexts():
        texts = stx_doc.getColorizableTexts()
        for text in texts:
            if not has_sub:                            
                append( text + NEOPORTAL_PARAGRAPH_MARKER )
            else:
                append( text )
                
    if stx_doc.getSubparagraphs():
        for sub in stx_doc.getSubparagraphs():
            flattenSTDoc( sub, append )

def getListOfParagraphs( doc  ):
    """
        Return list of paragraphs
    """

    stx_doc = ST.Basic( doc )
    #NP_Doc = NeoPortalSTDocument()
    #stx_doc = NP_Doc( doc )
    
    p_list = []
    
    flattenSTDoc ( stx_doc, append=p_list.append)
    
    return '\n\n'.join( p_list ).split( NEOPORTAL_PARAGRAPH_MARKER )

def getDTML(name ):
    """
        Save some typing with globals()
    """
    
    return DTMLFile( name, globals() )       

def addDTML( obj, id, title, fname, overwrite=0 ):
    """
        Add a DTML document from filesystem
    """
    
    f = open( fname )     
    file = f.read()
    f.close() 
    
    if overwrite:
        try:
            obj.manage_delObjects( [id + '._BACKUP',] )
        except:
            pass
        try:
            obj._setObject( id + '._BACKUP', getattr( aq_base( obj ),  id) )
        except:
            pass

        try:
            obj.manage_delObjects( [id,] )
        except:
            pass
            
    obj.manage_addDTMLMethod( id,title,file )
    return getattr( obj,id )      

def addZPT( obj,  id, title, fname, overwrite=0 ):
    """
        Add a ZPT document from filesystem
    """    
    
    f = open( fname )     
    file = f.read()
    f.close() 

    if overwrite:
        try:
            obj.manage_delObjects( [id + '._BACKUP'] )
        except:
            pass
        try:
            obj._setObject( id + '._BACKUP', getattr( aq_base( obj ),  id ) )
        except:
            pass

        try:
            obj.manage_delObjects( [id, ] )
        except:
            pass    
    manage_addPageTemplate( obj, id )
    obj._getOb( id ).write( file )
    
    return getattr( obj, id )

def addImage( obj,  id, title, fname, overwrite=0 ):  
    """
        Add an Image from filesystem
    """
        
    f = open( fname,'rb' )     
    
    img = f.read()
    f.close()     
    tlen = len( img )     

    if overwrite:
        try:
            obj.manage_delObjects( [id + '._BACKUP'] )
        except:
            pass
        try:
            obj._setObject( id + '._BACKUP', getattr( aq_base( obj ),  id ) )
        except:
            pass

        try:
            obj.manage_delObjects( [id, ] )
        except:
            pass    
            
    new_id = obj.manage_addImage( id,img,title )
    
def addPyScript( obj,  id, fname, overwrite=0 ):
    """
        Add a Python Script from filesystem
    """
        
    f = open( fname )
    file = f.read()
    f.close() 

    if overwrite:
        try:
            obj.manage_delObjects( [id + '._BACKUP'] )
        except:
            pass
        try:
            obj._setObject( id + '._BACKUP', getattr( aq_base( obj ),  id ) )
        except:
            pass

        try:
            obj.manage_delObjects( [id, ] )
        except:
            pass    
    manage_addPythonScript( obj, id )
    obj._getOb(id).write(file)
    
    return getattr( obj, id )  

def setFormMsg( req, msg ):
    """
        Cook REQUEST to have a form message
    """
    
    req.set( 'portal_status_message', msg )
    
    return req
    
def setFormError( req, key, msg ):
    """
        Cook REQUEST to have a form error message
    """
    
    errors = req.get('errors', {})

    errors[key] = msg
    
    req.set( 'errors', errors )   
    
    return req
        
def cutString( str, chunk=10, ellipsis='...' ):
    """
        Return contracted string
    """
    
    if len(str) <= chunk or not chunk:
        return str      
        
    match = re.compile('\s*|\w*' ).match( str, chunk )
    if match is None:

        ret_size = chunk
    
        return str[:ret_size] + ellipsis
    else:
        ret_size = match.start() - 1
        cpos = ret_size
        try:
            # Asian-friendly
            while( ord( str[cpos]) & 0x80 ):
                cpos = cpos - 1
        except:
            pass
            
        if cpos > 0:
            ret_size = cpos
                    
        return str[:ret_size] + ellipsis

def isEmailValid( email, bad_domains='' ):
    """
        Validate email address
    """
    
    if not email:
        return 0
    
    if type( bad_domains ) == type( [] ) and len( bad_domains ) > 0:
        for dom in bad_domains:
            if re.compile( '@' + dom ).search( email ) is not None:
                return 0

    if re.compile('\s' ).search( email ) is not None:
        return 0

    if re.compile( r'^[_\-\.0-9a-z]+@([0-9a-z][_\-0-9a-z\.]+)\.([a-z]{2,4}$)', re.IGNORECASE ).search( email ) is None:
        return 0
    
    return 1

def removeTags( text ):
    """
        Remove any markup tags
    """
    # remove tags
    text = re.sub('<[^<>]*>', '', text, re.DOTALL + re.MULTILINE)
    
    # bogus tags
    text = re.sub('\&lt;.*?\&gt;', '', text, re.DOTALL + re.MULTILINE)
    
    return text

def natSortElements( list ):
    """ Natural-sorts page_1, page_11, page_2... list."""

    list.sort(lambda x,y: cmp(int(x.getId().split('_' )[1]), int(y.getId().split('_' )[1])))
    
    return list
                       

###################################################################################
# 
#       Bogus Expression and ActionInformation Class
#
###################################################################################

from Globals import Persistent
from Products.PageTemplates.Expressions import getEngine

try:
    from Products.CMFCore.Expression import Expression as BaseExpression
    from Products.CMFCore.ActionInformation import ActionInformation as BaseAction

except ImportError:
    
    from OFS.SimpleItem import SimpleItem
    
    class BaseExpression ( Persistent ):
        """
            Fake Expression for CMF compatibility and easy migration: modeled after CMF Expression
        """        
        text = ''
        _v_compiled = None
    
        security = ClassSecurityInfo()
    
        def __init__( self, text ):
            self.text = text
            self._v_compiled = getEngine().compile( text )
    
        def __call__( self, econtext ):
            compiled = self._v_compiled
            if compiled is None:
                compiled = self._v_compiled = getEngine().compile( self.text )
            res = compiled( econtext )
            if isinstance( res, Exception ):
                raise res
            return res
    
    class BaseAction( SimpleItem ):
        """
            Fake ActionInformation for CMF compatibility and easy migration: modeled after CMF's ActionInformation
        """
        
        _isActionInformation = 1
        __allow_access_to_unprotected_subobjects__ = 1
    
        security = ClassSecurityInfo()
    
        def __init__( self
                    , id
                    , title=''
                    , description=''
                    , category='object'
                    , condition=''
                    , permissions=()
                    , priority=10
                    , visible=1
                    , action=''
                    ):
            if condition and type( condition ) == type( '' ):
                condition = BaseExpression( condition )
    
            if action and type( action ) == type( '' ):
                action = BaseExpression( action )
    
            self.id = id
            self.title = title
            self.description = description
            self.category = category 
            self.condition = condition
            self.permissions = permissions
            self.priority = priority 
            self.visible = visible
            self.action = action
    
        security.declarePublic('Title' )
        def Title( self ):
            """
                Return Title
            """
            return self.title or self.getId()
    
        security.declarePublic('Description' )
        def Description( self ):
            """
                Return Description
            """
            return self.description
    
        security.declarePrivate('testCondition' )
        def testCondition( self, ec ):
            if self.condition:
                return self.condition(ec)
            else:
                return 1
    
        security.declarePublic( 'getAction' )
        def getAction( self, ec ):
            """
                Return action
            """
            info = {}
            info['id'] = self.id
            info['name'] = self.Title()
            info['url'] = self.action and self.action( ec ) or ''
            info['permissions'] = self.getPermissions()
            info['category'] = self.getCategory()
            info['visible'] = self.getVisibility()
            return info 
    
        security.declarePublic( 'getActionExpression' )
        def getActionExpression( self ):
            """ 
                Return the text of the TALES expression for our URL.
            """
            action = self._getActionObject()
            return action and action.text or ''
    
        security.declarePublic( 'getCondition' )
        def getCondition( self ):
            """ 
                Return the text of the TALES expression for our condition.
            """
            return getattr( self, 'condition', None ) and self.condition.text or ''
    
        security.declarePublic( 'getPermissions' )
        def getPermissions( self ):
    
            """ Return the permission, if any, required to execute the action.
    
            Return an empty tuple if no permission is required.
            """
            return self.permissions
    
        security.declarePublic( 'getCategory' )
        def getCategory( self ):
            """ 
                Return the category in which the action should be grouped.
            """
            return self.category or 'object'
    
        security.declarePublic( 'getVisibility' )
        def getVisibility( self ):
            """ 
                Return if action visible
            """
            return self.visible
    
        security.declarePrivate( 'clone' )
        def clone( self ):
            """ 
                Return my clone
            """
            return self.__class__( id=self.id
                                 , title=self.title
                                 , description=self.description
                                 , category =self.category
                                 , condition=self.getCondition()
                                 , permissions=self.permissions
                                 , priority =self.priority
                                 , visible=self.visible
                                 , action=self.getActionExpression()
                                 )

NeoPortalBaseExpression = BaseExpression
NeoPortalBaseAction = BaseAction

# borrowed from Plone
class IndexIterator:
    __allow_access_to_unprotected_subobjects__ = 1 

    def __init__(self, upper=10000):
        self.upper=upper
        self.pos=0

    def next(self):
        if self.pos <= self.upper:
            self.pos += 1
            return self.pos
        raise KeyError, 'Reached upper bounds'

###################################################################################
# 
#       Korean-specific methods
#
###################################################################################

try:
    import codecs
    (UTF8_encode, UTF8_decode, UTF8_streamreader, UTF8_streamwriter) = codecs.lookup( 'UTF-8' )
    (EUCKR_encode, EUCKR_decode,EUCKR_streamreader, EUCKR_streamwriter) = codecs.lookup( 'korean.python.euc-kr' )
    
    def euckr_to_utf8( msg ):
        ( umsg, ulen ) = UTF8_encode( unicode( msg, "korean.python.euc-kr" ) )
        return umsg
    
    def utf8_to_euckr( msg ):
        ( umsg, ulen ) = UTF8_decode(msg)
        ( euc_msg, byte_used ) = EUCKR_encode( umsg )
        
        return euc_msg
except:
    def euckr_to_utf8( msg ):
        return msg

    def utf8_to_euckr( msg ):
        return msg
