#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: test_NPL.py,v 1.1.1.1 2002/12/13 01:57:01 wankyu Exp $
Description

 Test NeoPortal Library
"""

import Zope

from ZPublisher.HTTPRequest import HTTPRequest, HTTPResponse, FileUpload
from sys import stdin
from os import environ
environ['SERVER_NAME'] = 'NeoPortal Library Test Server'
environ['SERVER_URL'] = 'http://www.zoper.net/'
environ['SERVER_PORT'] = '8080'
REQUEST = HTTPRequest(stdin, environ, HTTPResponse())    

REQUEST['SERVER_URL'] = 'http://www.zoper.net'

from unittest import TestCase, TestSuite, makeSuite, main

from Globals import package_home
from Products.NeoPortalLibrary.NP_ExternalFile import addNeoPortalExternalFile
from Products.NeoPortalLibrary.NP_Mailer import NeoPortalMailer

from OFS.Folder import Folder

import os, re

ARCHIVE_FILE_PATH = package_home(globals())+ 'test_archive/'
ARCHIVE_FILE_PATH = ARCHIVE_FILE_PATH.replace('__main__', '')
mail_from = 'NeoPortal Library Test Suite ѱ۵ ׽Ʈ<wankyu@neoqst.com>'
mail_to = 'ֿϱ<wankyu@neoqst.com>'

class NPL_Tests( TestCase ):
    
    #setUp/tearDown
    def setUp( self ):
        """
            Add root Folder for testing
        """
        
        # add a test folder
        self.testFolder = Folder(id='testFolder')
        self.testFolder.REQUEST = REQUEST

        if not os.path.isdir(ARCHIVE_FILE_PATH ):
            os.mkdir(ARCHIVE_FILE_PATH, 0755)        
            
    def getRootFolder( self ):
        return self.testFolder
        
    def tearDown( self ):
        try:
            os.unlink(ARCHIVE_FILE_PATH + 'test_file.txt')
            os.removedirs(ARCHIVE_FILE_PATH)
        except:
            pass

    def test_ExternalFile( self ):
        
        test_file = open('./test_file.gif', 'rb')
        addNeoPortalExternalFile( self.getRootFolder()
                        , filepath=ARCHIVE_FILE_PATH + 'test_file.gif'
                        , id='testFile'
                        , title='Test File'
                        , description='This is a test file.'
                        , check_filepath=0
                        , file=test_file)
        
        ext_file = self.getRootFolder()._getOb('testFile')

        self.failUnless( len(ext_file) > 0 ,  "Failed to upload NeoPortal External File!")
        self.failUnless( ext_file.isImage() ,  "Image is not recognized by NeoPortal External File!")

    def test_Mailer( self ):
        
        test_file1 = './test_file.gif'
        test_file2 = './test_file2.gif'
        test_file3 = './test_file3.txt'
        html_file  = open('./test_file4.html', 'rb')

        attachments = [ test_file1, test_file2, test_file3 ]
        
        mailer = NeoPortalMailer( mailhost=None
                    , from_addr=mail_from
                    , to_addr=mail_to
                    , reply_addr=''
                    , cc_addr='neobundy@dreamwiz.com'
                    , subject='NeoPortal Mailer Test  ϴ '
                    , description=''
                    , body=html_file.read()
                    , attachments=attachments
                    , html=1
                    , charset='utf-8'
                    , encoding='base64' )        
        mailer.send()

def test_suite( ):
    return TestSuite((
        makeSuite( NPL_Tests ),
        ))

if __name__ == '__main__':
    main(defaultTest='test_suite')